option (null:hard);

module http_api {

string default_status_message(integer status_code)
{
	switch (status_code) {
	case 100: return "Continue";
	case 101: return "Switching Protocols";
	case 102: return "Processing";
	case 103: return "Early Hints";
	case 200: return "OK";
	case 201: return "Created";
	case 202: return "Accepted";
	case 203: return "Non-Authoritative Information";
	case 204: return "No Content";
	case 205: return "Reset Content";
	case 206: return "Partial Content";
	case 207: return "Multi-Status";
	case 208: return "Already Reported";
	case 226: return "IM Used";
	case 300: return "Multiple Choices";
	case 301: return "Moved Permanently";
	case 302: return "Found";
	case 303: return "See Other";
	case 304: return "Not Modified";
	case 305: return "Use Proxy";
	case 306: return "Switch Proxy";
	case 307: return "Temporary Redirect";
	case 308: return "Permanent Redirect";
	case 400: return "Bad Request";
	case 401: return "Unauthorized";
	case 402: return "Payment Required";
	case 403: return "Forbidden";
	case 404: return "Not Found";
	case 405: return "Method Not Allowed";
	case 406: return "Not Acceptable";
	case 407: return "Proxy Authentication Required";
	case 408: return "Request Timeout";
	case 409: return "Conflict";
	case 410: return "Gone";
	case 411: return "Length Required";
	case 412: return "Precondition Failed";
	case 413: return "Payload Too Large";
	case 414: return "URI Too Long";
	case 415: return "Unsupported Media Type";
	case 416: return "Range Not Satisfiable";
	case 417: return "Expectation Failed";
	case 418: return "I'm a teapot";
	case 421: return "Misdirected Request";
	case 422: return "Unprocessable Entity";
	case 423: return "Locked";
	case 424: return "Failed Dependency";
	case 425: return "Too Early";
	case 426: return "Upgrade Required";
	case 428: return "Precondition Required";
	case 429: return "Too Many Requests";
	case 431: return "Request Header Fields Too Large";
	case 451: return "Unavailable For Legal Reasons";
	case 500: return "Internal Server Error";
	case 501: return "Not Implemented";
	case 502: return "Bad Gateway";
	case 503: return "Service Unavailable";
	case 504: return "Gateway Timeout";
	case 505: return "HTTP Version Not Supported";
	case 506: return "Variant Also Negotiates";
	case 507: return "Insufficient Storage";
	case 508: return "Loop Detected";
	case 510: return "Not Extended";
	case 511: return "Network Authentication Required";
	default: throw (E_UNSPECIFIC, strcat("Non standard status code: ", str(status_code)));
	}
}

http.response mk_response(integer status_code)
{
	return new http.response(status_code, default_status_message(status_code));
}

http.response __mk_json_response(integer status_code, string body, logical deflate)
{
	http.response resp = new http.response(status_code, default_status_message(status_code));
	resp.set_header("Content-Type", "application/json");

	if (deflate) {
		resp.set_header("Content-Encoding", "gzip");
		resp.set_body(zlib.deflate(new blob(body), zlib.format.FMT_GZIP));
	} else {
		resp.set_body(body);
	}

	return resp;
}

http.response mk_deflated_response(integer status_code, json.node jn, integer indent = -1) =
	__mk_json_response(status_code, indent < 0 ? jn.print() : jn.print(indent), true);

http.response mk_deflated_response(integer status_code, json.builder jb) =
	__mk_json_response(status_code, jb.get(), true);

http.response mk_response(integer status_code, json.node jn, integer indent = -1) =
	__mk_json_response(status_code, indent < 0 ? jn.print() : jn.print(indent), false);

http.response mk_response(integer status_code, json.builder jb) =
	__mk_json_response(status_code, jb.get(), false);

http.response mk_response_raw(
	integer status_code,
	string option(nullable) status_msg,
	string option(nullable) content_type,
	string option(nullable) body)
{
	if (null(status_msg))
		status_msg = default_status_message(status_code);

	http.response resp = new http.response(status_code, status_msg);

	if (!null(content_type))
		resp.set_header("Content-Type", content_type);

	if (!null(body))
		resp.set_body(body);

	return resp;
}

http.response mk_error_response(integer status_code, string msg)
{
	json.builder jb = new json.builder();
	jb.begin_object();
	jb.begin_member("error");
	jb.append(msg);
	jb.end_object();
	return __mk_json_response(status_code, jb.get(), false);
}

http.response mk_method_not_allowed_response(vector(string) allowed_methods)
{
	if (v_size(allowed_methods) == 0)
		throw (E_UNSPECIFIC, "Must allow some method for mk_method_not_allowed_response");

	integer n = v_size(allowed_methods);
	vector(http.method) methods[n];
	for (integer i = 0; i < n; ++i) {
		str_to_enum(allowed_methods[i], methods[i]);
	}

	integer status_code = 405;
	http.response resp = new http.response(status_code, default_status_message(status_code));
	resp.set_header("Allow", str_join(enum_id(methods), ", "));
	return resp;
}

json.builder mk_json(string name, string value, integer indent = 0)
{
	json.builder jb = new json.builder(indent);
	jb.begin_object();
	jb.begin_member(name);
	jb.append(value);
	jb.end_object();
	return jb;
}
json.builder mk_json(string name, vector(string) value, integer indent = 0)
{
	json.builder jb = new json.builder(indent);
	jb.begin_object();
	jb.begin_member(name);
	jb.append(value);
	jb.end_object();
	return jb;
}
json.builder mk_json(string name, vector(number) value, integer indent = 0)
{
	json.builder jb = new json.builder(indent);
	jb.begin_object();
	jb.begin_member(name);
	jb.append(value);
	jb.end_object();
	return jb;
}
json.builder mk_json(string name, vector(integer) value, integer indent = 0)
{
	json.builder jb = new json.builder(indent);
	jb.begin_object();
	jb.begin_member(name);
	jb.append(value);
	jb.end_object();
	return jb;
}
json.builder mk_json(string name, vector(date) value, integer indent = 0)
{
	json.builder jb = new json.builder(indent);
	jb.begin_object();
	jb.begin_member(name);
	jb.append(value);
	jb.end_object();
	return jb;
}


number get_json_number(json.node node, string member, number option(nullable) dflt)
{
	if (!node.has_object_member(member))
		return dflt;
	return node.get_object_member(member).get_number();
}

number get_json_number(json.node node, string member)
{
	return node.get_object_member(member).get_number();
}

string get_json_string(json.node node, string member, string option(nullable) dflt)
{
	if (!node.has_object_member(member))
		return dflt;
	return node.get_object_member(member).get_string();
}

string get_json_string(json.node node, string member)
{
	return node.get_object_member(member).get_string();
}

} // module http_api
