option(null: hard);	

/*	
	instrument wrapper functions
	Developer: Algorithmica Research, Magnus Nyström

	--------------------------------------------------------------------
	 	swap_fixflt create funcs
	--------------------------------------------------------------------			
*/

//--------
// from db
//--------


/*-----------------------------------------------------------------------
  swap_fixflt
  no input of settle_date for db swaps
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt(instrument_name 			instrument_name, 
							date option(nullable) 		trade_date 	= null<date>,
							string option(nullable) 	quote_side 	= null<string>,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{			
		__instrument c = __instrument(instrument_name,trade_date,quote_side, null<date> );		
		return create_swap_fixflt(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt");
		return null<swap_fixflt>;
	}
}

/*-----------------------------------------------------------------------
  swap_fixflt
  no input of settle_date for db swaps
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt(instrument_name 			instrument_name, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{				
		__instrument c = __instrument(instrument_name,trade_date,quote, null<date> );
		return create_swap_fixflt(c,error, E_INIT);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt");
		return null<swap_fixflt>;
	}
}


/*-----------------------------------------------------------------------
  swap_fixflt
  no input of settle_date for db swaps
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt(instr_def 					instr_def, 
							date option(nullable) 		trade_date 	= null<date>,
							string option(nullable) 	quote_side 	= null<string>,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{				
		__instrument c = __instrument(instr_def,trade_date,quote_side, null<date> );
		return create_swap_fixflt(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt");
		return null<swap_fixflt>;
	}
}
/*-----------------------------------------------------------------------
  swap_fixflt
  no input of settle_date for db swaps
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt(instr_def 					instr_def, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{			
		__instrument c = __instrument(instr_def,trade_date,quote, null<date> );
		return create_swap_fixflt(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt");
		return null<swap_fixflt>;
	}
}


//----------------
// from db or not
//----------------

/*-----------------------------------------------------------------------
  func: swap_fixflt
  arguments: 
	instr_def: 	can be created either from the db or user defined 				 
				instr_def is copied and modified
				no lookup of quotes
	instrument_name: user defined name

  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt(	instr_def 				instr_def, 
							string 					instrument_name  ,
							date option(nullable) 	trade_date ,
							date option(nullable) 	settle_date ,//=effective_date=start_date							
							date  option(nullable) 	maturity,//maturity has precedence over mat_code
							string option(nullable) maturity_code,
							number option(nullable)	fix_rate,							
							number 					nominal 	= 100,
							error_info option(nullable) error	= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{			
		__instrument c;

		if(!null(maturity))
			c = __instrument_fixflt_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity,fix_rate,nominal );
		else if(!null(maturity_code))
			c = __instrument_fixflt_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity_code,fix_rate,nominal );
		else
			QL_FAIL("invalid maturity/maturity_code");			

		return create_swap_fixflt( c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt");
		return null<swap_fixflt>;
	}
}


//----------------
// no db, swaplib version
// an external swap is created. Note: an external swap has no internal instr_def
// but can be created from an instr_def
//----------------

/*-----------------------------------------------------------------------
  func: swap_fixflt
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt(ql_fixed_income_swap ql_fixed_income_swap,
						error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{	
		QL_FAIL_COND(CORE_INT.swap_instr_type(ql_fixed_income_swap) != instr_type.SWAP_FIXFLT, "invalid instrument type", E_INIT);

		__instrument c = __instrument_swap(ql_fixed_income_swap);

		return create_swap_fixflt(c, error);				
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt");
		return null<swap_fixflt>;
	}
}


/*-----------------------------------------------------------------------
  func: swap_fixflt
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt(ql_fix_leg 		ql_fix_leg, 							
						ql_float_leg 	ql_float_leg,
						string 			id,
						error_info option(nullable) error = null<error_info>)
option (category: 'Curve and Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{			
		__instrument c = __instrument_swap(ql_fix_leg,ql_float_leg,id);

		return create_swap_fixflt(c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt");
		return null<swap_fixflt>;
	}
}

module CORE_INT_FIXFLT
{
	void swap_start_date_helper(date_code option(nullable) 	spot_code,
								date option(nullable) 		trade_date,
								calendar option(nullable) 	pmt_calendar,
								string option(nullable) 	fwd_start_code,
								out date 					start_date,
								out date 					settle_date)
	{
		try {
			if(null(settle_date) && !null(trade_date)){
				settle_date = spot_code.apply_fwd(trade_date,pmt_calendar);
				if(!null(start_date) && start_date > settle_date)
					settle_date = start_date;
			}//this first part seems redundant?
				
			if(null(start_date)){			
				if(!null(fwd_start_code)){
					QL_REQUIRE(!null(trade_date),"invalid trade date");
					date spot_date 			= spot_code.apply_fwd(trade_date,pmt_calendar);				
					date_code start_code 	= date_code(fwd_start_code);
					start_date 				= start_code.apply_fwd(spot_date,pmt_calendar);
					start_date				= pmt_calendar.adjust_date (start_date, bd_convention.BD_FOLLOWING);
					settle_date 			= start_date;
				}
				else if(!null(settle_date))
					start_date = settle_date;
				else if(!null(trade_date)){
					settle_date = spot_code.apply_fwd(trade_date,pmt_calendar);
					start_date = settle_date;
				}
				/*possible that settle_date and start_date ends up being null*/
			}
			else {
				if(null(settle_date) || start_date > settle_date) {
					settle_date = start_date;
				}				
			}
		}
		catch {
			start_date = settle_date = null<date>;
		}
	}

	tenor_code swap_maturity_helper(instr_def option(nullable)	def,
									date option(nullable) maturity,
									string 	option(nullable) maturity_code)
	{		
		if(!null(maturity))
			return null<tenor_code>;	
		else if(!null(maturity_code))
			return tenor_code(maturity_code);		

		if(null(def))
			QL_FAIL("missing maturity/maturity_code", E_INIT);
		
		error_info ee = error_info(true,true);
		string mc = def.maturity_code(ee);
		if(!null(mc))
			return tenor_code(mc);
		
		QL_FAIL("missing maturity/maturity_code", E_INIT);			
	}

	void unpack_instr_def(	instr_def 			def,
							out logical 		is_ccy,							
							out notional_exchg_style ne,
							out date_code 		settle_code ,	
							out coupon_freq  	fix_cpn_freq ,
							out string 			fix_ccy,
							out calendar 		fix_pmt_calendar,
							out bd_convention 	fix_pmt_bus_day,
							out logical			fix_end_of_month,
							out day_count_method fix_dc,
														
							out flt_reset_style flt_reset_style,
							out coupon_freq  	flt_cpn_freq,
							out string 			flt_ccy,
							out calendar 		flt_pmt_calendar,
							out bd_convention 	flt_pmt_bus_day,
							out logical			flt_end_of_month,
							out day_count_method flt_dc,
							out reset_freq  	flt_r_freq,
							out ir_index 		flt_ir_index,
							out flt_comp_avg_type 		flt_ca_t,
							out flt_sprd_comp_method 	flt_comp_m,
							out flt_avg_method 			flt_avg)
	{
		
		QL_FAIL_COND(!def.is_swap_fixflt(), "instr_def not a fix-float swap", E_INIT);
		is_ccy = def.is_ccy_swap();
		error_info ee = error_info(true,true);

		ne = def.swap_not_exchange(ee);
		if(null(ne))
			ne 	= is_ccy ? NE_BOTH : NE_NONE;
		
		settle_code 	= date_code(def.settle_code());		
		fix_cpn_freq 	= coupon_freq_from_int(def.coupon_freq(ee))  ;		
		fix_pmt_calendar= def.swap_calendar_leg1(ee);
		fix_pmt_bus_day = def.swap_pmt_bus_day_leg1(ee);
		fix_end_of_month= def.swap_eom_leg1(ee);		
		fix_ccy 		= def.swap_currency_leg1(ee);
		fix_dc 			= def.swap_dc_method_leg1(ee);
		
		flt_pmt_calendar= def.swap_calendar_leg2(ee);
		flt_pmt_bus_day = def.swap_pmt_bus_day_leg2(ee);
		flt_end_of_month= def.swap_eom_leg2(ee);
		flt_ccy 		= def.swap_currency_leg2(ee);
		flt_dc 			= def.swap_dc_method_leg2(ee);
		flt_reset_style = RS_NORM;
		flt_cpn_freq 	= coupon_freq_from_int(def.swap_coupon_freq_leg2(ee))  ;
		flt_r_freq 		= reset_freq_from_int(def.swap_reset_freq_leg2(ee))  ;
		flt_ir_index 	= def.swap_ir_index_leg2(ee);
		flt_ca_t		= def.swap_comp_avg_type_leg2(ee);
		flt_comp_m 		= def.swap_sprd_comp_method_leg2(ee);
		flt_avg 		= def.swap_avg_method_leg2(ee);
	}

	void set_ccy(logical is_ccy,
				 number fix_notional,
				 out number flt_notional,
				 out number fix_fx_rate ,
				 out number flt_fx_rate)
	{
		if(is_ccy) {
			
			if(null(fix_fx_rate)) 
				fix_fx_rate = 1;			
			if(null(flt_fx_rate)) 
				flt_fx_rate = 1;
			if(null(flt_notional) ) 
				flt_notional = fix_notional*fix_fx_rate/flt_fx_rate;
		}
		else {
			
			flt_notional		= fix_notional;
			fix_fx_rate 		= 1;
			flt_fx_rate 		= 1;
		}
	}
	void set_plain_vanilla(	number fix_notional	,
							disc_func option(nullable) fix_df,
							disc_func option(nullable) flt_df,
							number option(nullable) curr_fix,
							logical is_ccy,
							out date  fix_eff_date 		,
							out date  flt_eff_date 		,
							out date  fix_first_cpn_date,
							out date  fix_last_reg_date ,
							out date  flt_first_cpn_date,
							out date  flt_last_reg_date ,
							out number	flt_spread 		,
							out integer	cpn_roll_day 	,
							out ..disc_func fix_disc_func,
							out ..disc_func flt_disc_func,							
							out number flt_notional		,
							out number fix_fx_rate 		,
							out number flt_fx_rate 		,
							out vector(date) fixing_dates,	
							out vector(number) fixing_rates,
							out number fix_proxy ,
							out logical  allow_fwd_fix)
	{
		if(is_ccy) {
			QL_REQUIRE(!null(flt_df),"a separate disc_func for each leg must be input for cross currency swaps");
		}
		
		fix_eff_date = null<date>;
		flt_eff_date = null<date>;
		fix_first_cpn_date = null<date>;
		fix_last_reg_date = null<date>;
		flt_first_cpn_date = null<date>;
		flt_last_reg_date = null<date>;
		flt_spread 		= 0;		
		cpn_roll_day 	= 0;
		fix_disc_func = fix_df;
		flt_disc_func = null(flt_df) ? fix_df : flt_df;

		set_ccy(is_ccy,fix_notional, flt_notional, fix_fx_rate , flt_fx_rate);		
	
		fixing_dates = null<vector(date)>;	
		fixing_rates= null<vector(number)>;
		fix_proxy 		= curr_fix;
		allow_fwd_fix	= false;
	}

	void set_tenor_surface(date trade_date,
						   ..disc_func option(nullable) flt_disc_func,								
							fwd_func option(nullable) 	flt_fwd_func,
							..disc_func option(nullable) flt_fwd_disc_func,
						   ir_index flt_ir_index,
						   out tenor_surface flt_tenor_surface)
	{
		logical allow_extrap = false;		
		try{
			if(!null(flt_fwd_func))
				flt_tenor_surface = tenor_surface(trade_date, flt_disc_func,[flt_fwd_func],[flt_ir_index],allow_extrap);
			else
				flt_tenor_surface = tenor_surface(trade_date, flt_disc_func,[flt_fwd_disc_func],[flt_ir_index],allow_extrap);
		}
		catch {
			flt_tenor_surface = null<tenor_surface>;
		}
	}
	/*-----------------------------------------------------------------------
	  func: swap_fixflt_plain_int
	  minimal constructor for single curr "screen"-swap, based on a instr_def
	  ----------------------------------------------------------------------*/
	swap_fixflt swap_fixflt_plain_int(	instr_def 					def,
										string 						instrument_name,	
										date option(nullable) 		trade_date,	
										date option(nullable) 		start_date,//=effective_date
										string option(nullable) 	fwd_start_code,
										date 	option(nullable)	maturity,
										string 	option(nullable)	maturity_code,
										number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
										number 	 					notional,																										
										logical 					pay_fixed,
										..disc_func option(nullable) disc_func,								
										fwd_func option(nullable) 	flt_fwd_func,
										..disc_func option(nullable) flt_fwd_disc_func,
										number option(nullable) 	curr_fix,
										integer 					fix_r_rnd_dec,
										error_info option(nullable) error )
	{	
		try{
			
			
			logical is_ccy,fix_end_of_month,flt_end_of_month;
			string fix_ccy, flt_ccy ;
			day_count_method fix_dc, flt_dc;
			notional_exchg_style ne;
			date_code settle_code;
			coupon_freq  fix_cpn_freq, flt_cpn_freq;
			flt_reset_style flt_reset_style;
			calendar fix_pmt_calendar,flt_pmt_calendar;
			bd_convention fix_pmt_bus_day,flt_pmt_bus_day;
			reset_freq  flt_r_freq;
			ir_index flt_ir_index;
			flt_comp_avg_type flt_ca_t;
			flt_sprd_comp_method flt_comp_m;
			flt_avg_method 	flt_avg;
			
			unpack_instr_def(def,is_ccy,ne, settle_code,fix_cpn_freq,fix_ccy,fix_pmt_calendar, fix_pmt_bus_day, fix_end_of_month, fix_dc,														
							flt_reset_style, flt_cpn_freq, flt_ccy, flt_pmt_calendar, flt_pmt_bus_day, flt_end_of_month, flt_dc,
							flt_r_freq, flt_ir_index, flt_ca_t, flt_comp_m, flt_avg);
			
			
			QL_FAIL_COND(is_ccy, "constructor not applicable for currency swap", E_INIT);
			
			//---def---			
			ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", fix_ccy, settle_code, fix_cpn_freq,fix_dc,fix_pmt_bus_day,	fix_pmt_calendar,fix_end_of_month,ne);
			
			ql_swap_float_def flt_def 	= ql_swap_float_def("float_def",flt_ccy,settle_code,flt_cpn_freq,flt_r_freq,flt_dc,flt_pmt_bus_day,flt_pmt_calendar,
															flt_end_of_month,flt_ca_t,flt_comp_m,flt_avg,flt_ir_index,flt_reset_style,ne);

			//---leg init----
			date  fix_eff_date,flt_eff_date,fix_first_cpn_date,fix_last_reg_date ,flt_first_cpn_date,flt_last_reg_date ;	
			integer	cpn_roll_day;
			..disc_func fix_disc_func,flt_disc_func;
			number flt_spread, flt_notional,fix_fx_rate,flt_fx_rate,fix_proxy ;
			vector(date) fixing_dates ;	
			vector(number) fixing_rates;
			logical  allow_fwd_fix;

			number fix_notional = notional;
			set_plain_vanilla(	fix_notional,disc_func,null<disc_func>,curr_fix,is_ccy,fix_eff_date,flt_eff_date,fix_first_cpn_date,
								fix_last_reg_date,flt_first_cpn_date,flt_last_reg_date , flt_spread,cpn_roll_day ,
								fix_disc_func, flt_disc_func, flt_notional,fix_fx_rate,flt_fx_rate ,
								fixing_dates, fixing_rates, fix_proxy , allow_fwd_fix);

			//---common---
			date settle_date;
			swap_start_date_helper(settle_code,trade_date,fix_pmt_calendar,fwd_start_code,start_date,settle_date);
			QL_FAIL_COND(null(settle_date),"invalid start date", E_INIT);

			tenor_code mat_code = swap_maturity_helper(def,maturity, maturity_code);	

			//---legs---
			tenor_surface flt_tenor_surface;
			set_tenor_surface(trade_date, flt_disc_func, flt_fwd_func, flt_fwd_disc_func, flt_ir_index,flt_tenor_surface);
			
			
			number init_rate 	= null(fix_coupon_rate) ? 0 : fix_coupon_rate;
			
			ql_fix_leg fix 		= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,
											pay_fixed,fix_notional,fix_eff_date,fix_first_cpn_date,fix_last_reg_date,init_rate,	
											fix_disc_func,cpn_roll_day,fix_fx_rate);
			
			ql_float_leg flt 	= ql_float_leg(flt_def,"float_leg",trade_date,settle_date,start_date,maturity,mat_code,
													!pay_fixed,flt_notional,flt_eff_date,flt_first_cpn_date,flt_last_reg_date,flt_spread,flt_disc_func,	
													flt_tenor_surface,cpn_roll_day,flt_fx_rate,
													fixing_dates,fixing_rates,fix_proxy,allow_fwd_fix);
			
			ql_fixed_income_swap sw = ql_fixed_income_swap(fix,flt,instrument_name);

			if(null(fix_coupon_rate)) {
				number fix_r	= sw.solver(0,swap_solver_code.SS_FIX_RATE_LEG1,false);
				QL_FAIL_COND(null(fix_r),"error solving for fixed coupon rate", E_INIT);				
				if(fix_r_rnd_dec >= 0)
					fix_r = round(fix_r,fix_r_rnd_dec);
				fix 	= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,
											pay_fixed,fix_notional,start_date,fix_first_cpn_date,fix_last_reg_date,fix_r,	
											fix_disc_func,cpn_roll_day,fix_fx_rate);
				sw 		= ql_fixed_income_swap(fix,flt,instrument_name);

			}

			return swap_fixflt(sw, error);						
		}
		catch {
			CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt_plain");
			return null<swap_fixflt>;
		}
	}

	/*-----------------------------------------------------------------------
	  swap_fixflt_par
	  ----------------------------------------------------------------------*/
	
	/*swap_fixflt swap_fixflt(instr_class_name 	class_name, 
								instrument_name		instrument_name, 									
								date 				trade_date, 
								date				settle_date,
								date 				maturity,
								number				fix_rate,
								out instr_error option(nullable) error)
	
	{
		instr_def i_d 	= instr_def(class_name ,true , error);
		if(error.is_error()) return null<swap_fixflt>;
		swap_fixflt sw 	= ..swap_fixflt(	i_d, instrument_name,trade_date,settle_date, 
											maturity,fix_rate,100, error);
		if(error.is_error()) return null<swap_fixflt>;
	}*/

	/*-----------------------------------------------------------------------
	  Name :  	create_fixflt_par
	  Notes :          
	  Changes:
	  			Date    		Who     Detail
	  			?				MN		for backward compat
	  ----------------------------------------------------------------------*/
	swap_fixflt create_fixflt_par(	instr_class_name 	class_name,
									instrument_name		instrument_name, 									
									date 				trade_date, 
									date				settle_date,
									date 				maturity,
									number				fix_rate)
	{
		try{
			error_info ee = new error_info(true,false);
			instr_def i_d 	= instr_def(class_name , ee);
			
			//QL_FAIL_COND(error.is_error(), error.message());
			ee = new error_info(true,true);
			swap_fixflt sw 	= ..swap_fixflt(i_d, instrument_name,trade_date,settle_date,
												maturity,null<string>,fix_rate,100, ee);
			//QL_FAIL_COND(null(sw.yield()),"invalid yield");
			return sw;
		}
		catch {
			CORE_INT.write_warning_message(err.message());
			return null<swap_fixflt>;
		}
	}
	//-------------------------------------------------------
	// create_class_fixflt  for backward compat
	//-------------------------------------------------------
	swap_fixflt create_class_fixflt(string class_name)
	{
		try {
			date d 	= #2011-03-12;
			date s 	= #2011-03-14;	
			date m 	= #2011-04-14;
	
			error_info ee = error_info(true,false);
			string name = strcat(class_name,"_");
			instr_def i_d 	= instr_def(class_name , ee);			
			//QL_FAIL_COND(error.is_error(), error.message());
			ee = new error_info(true,true);
			swap_fixflt sw 	= ..swap_fixflt(i_d, name,d,s,m,null<string>,0.01,100, ee);					
			return sw;
		}
		catch {
			CORE_INT.write_warning_message(err.message());
			return null<swap_fixflt>;
		}
	}


	/*-----------------------------------------------------------------------
	  func: swap_fixflt_plain
	  minimal constructor for single curr "screen"-swap,
	  ----------------------------------------------------------------------*/
	swap_fixflt swap_fixflt_plain_int(	swap_fixflt 				sw_tmpl,
										string 						instrument_name,	
										date option(nullable) 		trade_date,
										date option(nullable) 		settle_date,	
										date option(nullable) 		start_date,
										string option(nullable) 	fwd_start_code,
										date 	option(nullable)	maturity,
										string 	option(nullable)	maturity_code,
										number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
										number 	 					notional,																										
										logical 					pay_fixed,
										..disc_func option(nullable) disc_func,								
										fwd_func option(nullable) 	flt_fwd_func,
										disc_func option(nullable) 	flt_fwd_disc_func,
										number option(nullable) 	curr_fix,
										integer 					fix_r_rnd_dec,
										logical 					reg_first_period,
										error_info option(nullable) error )
	{	
		try{
			
			logical is_ccy = sw_tmpl.is_ccy_swap();
			QL_FAIL_COND(is_ccy, "constructor not applicable for currency swap", E_INIT);

			error_info ee = error_info(true,true);
			
			logical fix_end_of_month,flt_end_of_month;
			string fix_ccy, flt_ccy ;
			day_count_method fix_dc, flt_dc;
			notional_exchg_style ne;
			date_code settle_code;
			coupon_freq  fix_cpn_freq, flt_cpn_freq;
			flt_reset_style flt_reset_style;
			calendar fix_pmt_calendar,flt_pmt_calendar;
			bd_convention fix_pmt_bus_day,flt_pmt_bus_day;
			reset_freq  flt_r_freq;
			ir_index flt_ir_index;
			flt_comp_avg_type flt_ca_t;
			flt_sprd_comp_method flt_comp_m;
			flt_avg_method 	flt_avg;

			if(!sw_tmpl.is_ext_swap(ee)) {
				unpack_instr_def(sw_tmpl.instr_def(ee),is_ccy,ne, settle_code,fix_cpn_freq,fix_ccy,fix_pmt_calendar, fix_pmt_bus_day, fix_end_of_month, fix_dc,														
								flt_reset_style, flt_cpn_freq, flt_ccy, flt_pmt_calendar, flt_pmt_bus_day, flt_end_of_month, flt_dc,
								flt_r_freq, flt_ir_index, flt_ca_t, flt_comp_m, flt_avg);				
			}
			else {										
				ne 				= notional_exchg_style.NE_NONE;
				settle_code 	= date_code(sw_tmpl.settle_code());
				integer cfreq_fix, cfreq_flt;
				sw_tmpl.coupon_freq(cfreq_fix, cfreq_flt,ee);				
				fix_cpn_freq	= coupon_freq_from_int(cfreq_fix)  ;
				flt_cpn_freq 	= coupon_freq_from_int(cfreq_flt)  ;				
				sw_tmpl.calendar(fix_pmt_calendar,flt_pmt_calendar,ee);							
				sw_tmpl.pmt_bus_day(fix_pmt_bus_day,flt_pmt_bus_day,ee);
				sw_tmpl.eom(fix_end_of_month,flt_end_of_month,ee);
				sw_tmpl.dc_method(fix_dc,flt_dc,ee);
				sw_tmpl.currency(fix_ccy,flt_ccy,ee);
				flt_reset_style = RS_NORM;
				flt_r_freq 		= reset_freq_from_int(sw_tmpl.reset_freq_flt_leg(ee))  ;
				flt_ir_index 	= sw_tmpl.ir_index_flt_leg(ee);
				flt_ca_t 	= null;//FIX_ME
				flt_comp_m 	= null;//FIX_ME
				flt_avg 	= null;//FIX_ME
			}
						
			ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", fix_ccy, settle_code, fix_cpn_freq,fix_dc, fix_pmt_bus_day, fix_pmt_calendar,fix_end_of_month,ne);		
			ql_swap_float_def flt_def 	= ql_swap_float_def("float_def",flt_ccy,settle_code,flt_cpn_freq,flt_r_freq,flt_dc,flt_pmt_bus_day,
															flt_pmt_calendar,flt_end_of_month,flt_ca_t,flt_comp_m,flt_avg, flt_ir_index,flt_reset_style,ne);
			//---common---
			swap_start_date_helper(settle_code,trade_date,fix_pmt_calendar,fwd_start_code,start_date,settle_date);
			QL_FAIL_COND(null(settle_date),"invalid start date", E_INIT);

			tenor_code mat_code = swap_maturity_helper(null<instr_def>,maturity, maturity_code);	

			//---leg----
			date  fix_eff_date = null<date>;
			date  flt_eff_date = null<date>;
			date  fix_first_cpn_date = null<date>;
			date  fix_last_reg_date = null<date>;
			date  flt_first_cpn_date = null<date>;
			date  flt_last_reg_date = null<date>;
			
			if(reg_first_period){
				
				schedule sched = schedule(	start_date,maturity,maturity_code,null<number>,fix_pmt_calendar,
											fix_pmt_bus_day,fix_pmt_bus_day,schedule_period_type.REGULAR,
											 schedule_period_type.NONE,fix_end_of_month,fix_eff_date,fix_first_cpn_date,fix_last_reg_date,0,0);

				fix_first_cpn_date 	= sched.first_date(false);
				fix_last_reg_date 	= sched.last_reg_date(false);
				fix_eff_date		= sched.start_date(true);
		
			}

			number	flt_spread 		= 0;		
			integer	cpn_roll_day 	= 0;
			..disc_func fix_disc_func = disc_func;
			..disc_func flt_disc_func = disc_func;
			
			vector(date) fixing_dates = null<vector(date)>;	
			vector(number) fixing_rates= null<vector(number)>;
			number fix_proxy 		= curr_fix;
			logical  allow_fwd_fix	= false;
			
			number fix_notional		= notional;
			number flt_notional;
			number fix_fx_rate,flt_fx_rate;
			set_ccy(is_ccy,fix_notional, flt_notional, fix_fx_rate , flt_fx_rate);

			tenor_surface flt_tenor_surface;
			set_tenor_surface(trade_date, flt_disc_func, flt_fwd_func, flt_fwd_disc_func, flt_ir_index,flt_tenor_surface);
							
			ql_fix_leg fix;
			if(null(fix_coupon_rate))
				fix 				= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,pay_fixed,fix_notional,
												fix_eff_date,fix_first_cpn_date,fix_last_reg_date,0, fix_disc_func,cpn_roll_day,fix_fx_rate);
			else
				fix 				= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,pay_fixed,fix_notional,
												fix_eff_date,fix_first_cpn_date,fix_last_reg_date,fix_coupon_rate, fix_disc_func,cpn_roll_day,fix_fx_rate);

			ql_float_leg flt 		= ql_float_leg(flt_def,"float_leg",trade_date,settle_date,start_date,maturity,mat_code,!pay_fixed,
													flt_notional,flt_eff_date,flt_first_cpn_date,flt_last_reg_date,flt_spread,flt_disc_func,	
													flt_tenor_surface,cpn_roll_day,flt_fx_rate, fixing_dates,fixing_rates,fix_proxy,allow_fwd_fix);
			
			ql_fixed_income_swap sw = ql_fixed_income_swap(fix,flt,instrument_name);

			if(null(fix_coupon_rate)) {
				number fix_r	= sw.solver(0,SS_FIX_RATE_LEG1,false);
				QL_FAIL_COND(null(fix_r),"error solving for fixed coupon rate", E_INIT);
				if(fix_r_rnd_dec >= 0)
					fix_r = round(fix_r,fix_r_rnd_dec);
				fix = ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,pay_fixed,
								fix_notional,fix_eff_date,fix_first_cpn_date,fix_last_reg_date,fix_r, fix_disc_func,cpn_roll_day,fix_fx_rate);
				sw 	= ql_fixed_income_swap(fix,flt,instrument_name);
			}

			return swap_fixflt(sw, error);						
		}
		catch {
			CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt_plain");
			return null<swap_fixflt>;
		}
	}
}

/*-----------------------------------------------------------------------
  func: swap_fixflt_plain
  minimal constructor for single curr "screen"-swap,
  this version: fwd_func input
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt_plain(	instr_def 					def,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,																													
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) disc_func,								
								fwd_func option(nullable) 	flt_fwd_func,																																			
							    number option(nullable) 	curr_fix,
							    integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{
	return CORE_INT_FIXFLT.swap_fixflt_plain_int(def,instrument_name,trade_date,start_date,fwd_start_code,maturity,maturity_code,
								 fix_coupon_rate ,notional, pay_fixed,disc_func,flt_fwd_func, null<disc_func>,
								 curr_fix, fix_r_rnd_dec,error );
}

/*-----------------------------------------------------------------------
  func: swap_fixflt_plain
  minimal constructor for single curr "screen"-swap,
  this version: disc_func  input
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt_plain(	instr_def 					def,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,																													
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) disc_func,								
								disc_func option(nullable) 	flt_fwd_disc_func,																																			
							    number option(nullable) 	curr_fix,
								integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{
	return CORE_INT_FIXFLT.swap_fixflt_plain_int(def,instrument_name,trade_date,start_date,fwd_start_code,maturity,maturity_code,
								 fix_coupon_rate ,notional, pay_fixed,disc_func,null<fwd_func>, flt_fwd_disc_func,
								 curr_fix, fix_r_rnd_dec,error );
}


/*-----------------------------------------------------------------------
  func: swap_fixflt_plain
  minimal constructor for single curr "screen"-swap,
  this version: template + fwd_func + fwd_start_code + maturity_code input
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt_plain(	INSTR_TMPL.swap_fixflt_def_tmpl	tmpl,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								string option(nullable) 	fwd_start_code,//if tmpl have a code it is ignored
								string option(nullable)		maturity_code,//if tmpl have a code it is ignored																												
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) disc_func,								
								fwd_func option(nullable) 	flt_fwd_func,																																			
							    number option(nullable) 	curr_fix,
								integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{
	try{		
		instr_def def = instr_def_swap_fixflt(tmpl,fwd_start_code,maturity_code,error);		
		if(!CORE_INT.check_instr_def(def))
			return null<swap_fixflt>;
		return swap_fixflt_plain(	def,instrument_name,trade_date,null<date>,fwd_start_code,
									null<date>,maturity_code,fix_coupon_rate ,
									notional,pay_fixed,disc_func,flt_fwd_func,curr_fix,fix_r_rnd_dec,error);
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt_plain");
		return null<swap_fixflt>;
	}
}
/*-----------------------------------------------------------------------
  func: swap_fixflt_plain
  minimal constructor for single curr "screen"-swap,
  this version: template + fwd_func + maturity date input
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt_plain(	INSTR_TMPL.swap_fixflt_def_tmpl	tmpl,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,//if tmpl have a code it is ignored
								date option(nullable)		maturity,//if tmpl have a code it is ignored																												
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) disc_func,								
								fwd_func option(nullable) 	flt_fwd_func,																																			
							    number option(nullable) 	curr_fix,
								integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{
	try{		
		instr_def def = instr_def_swap_fixflt(tmpl,error);		
		if(!CORE_INT.check_instr_def(def))
			return null<swap_fixflt>;
		return swap_fixflt_plain(	def,instrument_name,trade_date,start_date,null<string>,
									maturity,null<string>,fix_coupon_rate ,
									notional,pay_fixed,disc_func,flt_fwd_func,curr_fix,fix_r_rnd_dec,error);
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt_plain");
		return null<swap_fixflt>;
	}
}
/*-----------------------------------------------------------------------
  func: swap_fixflt_plain
  this version: using instr_def
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt_plain(	instr_def 				instr_def, 
								string 					instrument_name  ,			
								date  option(nullable) 	start_date,
								date  option(nullable)	maturity,
								integer					cpn_roll_day,
								date option(nullable) 	fix_eff_date,
					    		date  option(nullable)	fix_first_cpn_date,
					    		date  option(nullable)	fix_last_reg_date,														
								number  				fix_coupon_rate ,									
								number 	 				fix_notional,
								date option(nullable) 	flt_eff_date,
								date  option(nullable)	flt_first_cpn_date,
					    		date  option(nullable)	flt_last_reg_date,
								number					flt_spread,							
								number 	option(nullable) flt_notional,
		
								date option(nullable) 		trade_date ,
								date option(nullable) 		settle_date ,
								logical 					pay_fixed,
								disc_func option(nullable)	fix_disc_func,//note: separate leg disc_funcs even if single currency
								number option(nullable) 	fix_fx_rate,//as of settle
								fwd_func  option(nullable)	flt_fwd_func,	
								disc_func option(nullable)	flt_disc_func,//note: separate leg disc_funcs even if single currency						
								number option(nullable) 	flt_fx_rate,//as of settle						
								vector(date) option(nullable) 	fixing_dates,	
								vector(number) option(nullable) fixing_rates,
								//number option(nullable) first_fixing,
							    //number option(nullable) last_fixing,
							    number option(nullable) 	fix_proxy, 
							    logical  option(nullable) 	allow_fwd_fix,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{
		string fix_ccy, flt_ccy;
		date_code settle_code ;		
		coupon_freq  fix_cpn_freq,flt_cpn_freq  ;
		calendar fix_pmt_calendar,flt_pmt_calendar;
		bd_convention fix_pmt_bus_day,flt_pmt_bus_day;
		day_count_method fix_dc, flt_dc;
		logical	fix_end_of_month, flt_end_of_month;
		reset_freq  flt_r_freq ;
		flt_reset_style flt_reset_style;
		ir_index flt_ir_index ;
		flt_comp_avg_type flt_ca_t;
		flt_sprd_comp_method flt_comp_m ;
		flt_avg_method flt_avg ;
		logical is_ccy;
		notional_exchg_style ne;
		CORE_INT_FIXFLT.unpack_instr_def(instr_def,is_ccy,ne, settle_code,fix_cpn_freq,fix_ccy,fix_pmt_calendar, fix_pmt_bus_day, fix_end_of_month, fix_dc,														
							flt_reset_style, flt_cpn_freq, flt_ccy, flt_pmt_calendar, flt_pmt_bus_day, flt_end_of_month, flt_dc,
							flt_r_freq, flt_ir_index, flt_ca_t, flt_comp_m, flt_avg);
		//---def---
		
		error_info ee 				= error_info(true,true);
		
		ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", fix_ccy, settle_code, fix_cpn_freq, fix_dc,fix_pmt_bus_day,fix_pmt_calendar, fix_end_of_month,ne);
		ql_swap_float_def flt_def 	= ql_swap_float_def("float_def",flt_ccy,settle_code, flt_cpn_freq,flt_r_freq,flt_dc,flt_pmt_bus_day,
														flt_pmt_calendar,flt_end_of_month, flt_ca_t,flt_comp_m,flt_avg, flt_ir_index,flt_reset_style,ne);

		//----common-----
		CORE_INT_FIXFLT.swap_start_date_helper(settle_code,trade_date,fix_pmt_calendar,instr_def.fwd_start_code(ee),start_date,settle_date);
		QL_FAIL_COND(null(settle_date),"invalid start date", E_INIT);

		tenor_code mat_code = CORE_INT_FIXFLT.swap_maturity_helper(instr_def, maturity, null<string>);				

		//---legs----
		CORE_INT_FIXFLT.set_ccy(is_ccy,fix_notional, flt_notional, fix_fx_rate , flt_fx_rate);

		tenor_surface flt_tenor_surface;
		CORE_INT_FIXFLT.set_tenor_surface(trade_date, flt_disc_func, flt_fwd_func, null<disc_func>, flt_ir_index,flt_tenor_surface);

			
		ql_fix_leg fix  		= ql_fix_leg(	fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,mat_code,pay_fixed,fix_notional,
												fix_eff_date,fix_first_cpn_date,fix_last_reg_date,fix_coupon_rate,fix_disc_func,cpn_roll_day,fix_fx_rate);		

		ql_float_leg flt 		= ql_float_leg(	flt_def,"float_leg",trade_date,settle_date,start_date,maturity,mat_code,
														!pay_fixed,flt_notional,flt_eff_date,flt_first_cpn_date,flt_last_reg_date,
														flt_spread,flt_disc_func, flt_tenor_surface,cpn_roll_day,flt_fx_rate,
														fixing_dates,fixing_rates,fix_proxy,allow_fwd_fix);
		
		ql_fixed_income_swap sw = ql_fixed_income_swap(fix,flt,instrument_name);

		return swap_fixflt(sw, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt_plain");
		return null<swap_fixflt>;
	}
}


/*-----------------------------------------------------------------------
  func: swap_fixflt_plain
  this version: from scratch (no instr_def)
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt_plain(	string 					instrument_name  ,			
								date option(nullable) 	start_date,
								date  					maturity,
								integer					cpn_roll_day,
								bd_convention			pmt_bus_day,
								calendar				pmt_calendar,
								logical					end_of_month,
								date_code				settle_code,
								notional_exchg_style option(nullable)  not_exchange,
								date option(nullable) 	fix_eff_date,
					    		date  option(nullable)	fix_first_cpn_date,
					    		date  option(nullable)	fix_last_reg_date,						
								day_count_method		fix_day_count,																
								number  				fix_coupon_rate ,
								integer 				fix_coupon_freq ,									
								number 					fix_notional,
								string					fix_currency,
								sink_fund_style option(nullable) fix_sink_fund_style,	
								vector(date) option(nullable) fix_sink_fund_dates,
								vector(number) option(nullable) fix_sink_fund,
								date option(nullable) 	flt_eff_date,
								date  option(nullable)	flt_first_cpn_date,
					    		date  option(nullable)	flt_last_reg_date,
								day_count_method		flt_day_count,
								number					flt_spread,
								integer 				flt_coupon_freq ,	
								number 	option(nullable) flt_notional,
								string					flt_currency,
								ir_index 				flt_ir_index,
								sink_fund_style option(nullable) flt_sink_fund_style,	
								vector(date) option(nullable) flt_sink_fund_dates,
								vector(number) option(nullable) flt_sink_fund,
								
								date option(nullable) 		trade_date ,
								date option(nullable) 		settle_date ,
								logical 					pay_fixed,
								disc_func option(nullable) 	fix_disc_func,
								number option(nullable) 	fix_fx_rate,//as of settle
								fwd_func option(nullable) 	flt_fwd_func,	
								disc_func option(nullable) 	flt_disc_func,						
								number option(nullable) 	flt_fx_rate,//as of settle
								vector(date) option(nullable) 	fixing_dates,	
								vector(number) option(nullable) fixing_rates,
								//number option(nullable) first_fixing,
							    //number option(nullable) last_fixing,
							    number option(nullable) 	fix_proxy, 
							    logical  option(nullable) 	allow_fwd_fix,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{
		CORE_INT.reset_single(error);
		//---def---
		logical is_ccy				= fix_currency != flt_currency;

		if(null(not_exchange))
			not_exchange 			= is_ccy ? NE_BOTH : NE_NONE;		

		coupon_freq  fix_cpn_freq 	= coupon_freq_from_int(fix_coupon_freq)  ;

		ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", fix_currency, settle_code, fix_cpn_freq, fix_day_count,pmt_bus_day,pmt_calendar,end_of_month,not_exchange);
		
		flt_reset_style reset_style = RS_NORM;
		coupon_freq  flt_cpn_freq 	= coupon_freq_from_int(flt_coupon_freq)  ;		
				
		ql_swap_float_def flt_def 	= ql_swap_float_def("float_def",flt_currency,settle_code, flt_cpn_freq,null<reset_freq>,flt_day_count,pmt_bus_day,
														pmt_calendar,end_of_month, null<flt_comp_avg_type>,null<flt_sprd_comp_method>,null<flt_avg_method>,
														flt_ir_index,reset_style,not_exchange);
		//---leg---

		CORE_INT_FIXFLT.set_ccy(is_ccy,fix_notional, flt_notional, fix_fx_rate , flt_fx_rate);

		tenor_surface flt_tenor_surface;
		CORE_INT_FIXFLT.set_tenor_surface(trade_date, flt_disc_func, flt_fwd_func, null<disc_func>, flt_ir_index,flt_tenor_surface);

		CORE_INT_FIXFLT.swap_start_date_helper(settle_code,trade_date,pmt_calendar,null<string>,start_date,settle_date);
		QL_FAIL_COND(null(settle_date),"invalid start date", E_INIT);

		ql_fix_leg fix 			= ql_fix_leg(	fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,null<tenor_code>,
												pay_fixed,fix_notional,fix_eff_date,fix_first_cpn_date,fix_last_reg_date,fix_coupon_rate,	
												fix_disc_func,cpn_roll_day,fix_fx_rate, fix_sink_fund_style,fix_sink_fund_dates,fix_sink_fund);

		ql_float_leg flt 		= ql_float_leg(flt_def,"float_leg",trade_date,settle_date,start_date,maturity,null<tenor_code>,
												!pay_fixed,flt_notional,flt_eff_date,flt_first_cpn_date,flt_last_reg_date,flt_spread,flt_disc_func,	
												flt_tenor_surface,cpn_roll_day,flt_fx_rate,flt_sink_fund_style,flt_sink_fund_dates,flt_sink_fund,
												fixing_dates,fixing_rates,fix_proxy,allow_fwd_fix);
		
		ql_fixed_income_swap sw	= ql_fixed_income_swap(fix,flt,instrument_name);

		return swap_fixflt(sw, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixflt_plain");
		return null<swap_fixflt>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fixflt_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt_plain(	swap_fixflt 				sw,
								string 						instrument_name,	
								date option(nullable) 		trade_date,
								date option(nullable) 		settle_date,	
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,																												
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) disc_func,								
								fwd_func option(nullable) 	flt_fwd_func,																																			
							    number option(nullable) 	curr_fix,
 							    integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{
	return CORE_INT_FIXFLT.swap_fixflt_plain_int(sw,instrument_name,trade_date,settle_date,start_date,fwd_start_code,maturity,maturity_code,
								 fix_coupon_rate ,notional, pay_fixed,disc_func,flt_fwd_func, null<disc_func>,
								 curr_fix, fix_r_rnd_dec,false,error );
}

/*-----------------------------------------------------------------------
  func: swap_fixflt_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt_plain(	swap_fixflt 				sw,
								string 						instrument_name,	
								date option(nullable) 		trade_date,
								date option(nullable) 		settle_date,
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,																												
								number option(nullable)		fix_coupon_rate ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) disc_func,								
								disc_func option(nullable) 	flt_disc_func,																																			
							    number option(nullable) 	curr_fix,
							    integer 					fix_r_rnd_dec,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{
	return CORE_INT_FIXFLT.swap_fixflt_plain_int(sw,instrument_name,trade_date,settle_date,start_date,fwd_start_code,maturity,maturity_code,
								 fix_coupon_rate ,notional, pay_fixed,disc_func,null<fwd_func>, flt_disc_func,
								 curr_fix, fix_r_rnd_dec,false,error );
}

/*-----------------------------------------------------------------------
  func: swap_fixflt_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fixflt swap_fixflt_plain(	instrument 					fixflt_sw,															
								number 	 					notional,																										
								logical 					pay_fixed,
								..disc_func option(nullable) disc_func,								
								fwd_func option(nullable) 	flt_fwd_func,																																		
							    number option(nullable) 	curr_fix,							    
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{
	error_info ee 			= new error_info(true,true);
	swap_fixflt sw 			= swap_fixflt(fixflt_sw,ee);
	string instrument_name 	= sw.name(ee);	
	date trade_date 		= sw.trade_date(ee);
	date settle_date 		= sw.settle_date(ee);
	date start_date 		= sw.issue_date(ee);
	string fwd_start_code	= null<string>;
	date maturity 			= sw.maturity(ee);			
	string maturity_code 	= null<string>;																												
	number fix_coupon_rate 	= sw.quote(ee)/100.0;
	return CORE_INT_FIXFLT.swap_fixflt_plain_int(sw,instrument_name,trade_date,settle_date,start_date,fwd_start_code,maturity,maturity_code,
												fix_coupon_rate ,notional, pay_fixed,disc_func,flt_fwd_func, null<disc_func>,
												curr_fix, -1,false,error );
}


/*-----------------------------------------------------------------------
  carry		EXT SWAP: ok
  ----------------------------------------------------------------------*/
void carry(	swap_fixflt sw,
			date 		fwd_settle_date,			
			out number  carry,
			out number  run_carry,			
			error_info option(nullable) error )
{
	
	try {
		error_info ee 				= new error_info(true,true);
		date fwd_trade_date 		= null<date>;		
		logical use_ai_adj 			= true;
		
		number carry_upfront, carry_fwd,carry_upfront_ai_adj,carry_fwd_ai_adj;
		number npv_spot_fix_leg, npv_spot_flt_leg, npv_fwd_fix_leg, npv_fwd_flt_leg;
		number reinv_irim_cf_fix_leg, reinv_irim_cf_flt_leg, irim_cf_fix_leg, irim_cf_flt_leg;
		number 	npv_spot_ir_fix_leg,npv_spot_ir_flt_leg;
										
		sw.upfront_carry_all(fwd_trade_date,fwd_settle_date,
						carry_upfront,carry_fwd,carry_upfront_ai_adj,carry_fwd_ai_adj, npv_spot_fix_leg,npv_spot_flt_leg,
						npv_fwd_fix_leg,npv_fwd_flt_leg,reinv_irim_cf_fix_leg,
						reinv_irim_cf_flt_leg,irim_cf_fix_leg,irim_cf_flt_leg,
						npv_spot_ir_fix_leg,npv_spot_ir_flt_leg,ee);
		
		number sw_rate 			= sw.coupon_fix_leg(ee);
		disc_func df			= sw.disc_func_fix_leg(ee);
		tenor_surface ts 		= sw.tenor_surface_flt_leg(ee);
		tenor_code tc			= sw.index_tenor_code_flt_leg(ee) ;
		fwd_func ff				= ts.fwd_curve_f(tc);
		date start_date 		= use_ai_adj ? fwd_settle_date : sw.start_date();
		
		swap_fixflt sw_fwd 		= swap_fixflt_plain(sw, "fwd", sw.trade_date(ee),fwd_settle_date,start_date,
														null<string>,sw.maturity(),null<string>, sw_rate ,									
														sw.notional(), sw.pay_fix(),df,ff, null<number>,-1,ee);
				
		number _pv01			= sw_fwd.pv01(null<number>,null<number>, null<disc_func>,ee);
		
		//swap_fixflt sw_fwd = new swap_fixflt(this);
		//sw_fwd = sw_fwd.set_date(this.trade_date(ee),fwd_settle_date,ee);

		//number _pv01 = sw_fwd.pv01();

		carry 		= use_ai_adj ? carry_fwd_ai_adj : carry_fwd;
		run_carry 	= carry / _pv01;
		/*number cpn = this.coupon(ee);
		number fwd_npv = npv_fwd_fix_leg+npv_fwd_flt_leg;
		number rf = sw_fwd.imp_rate_fix_leg(fwd_npv,false,ee);
		number ccy = (rf-cpn)*10000;*/
			
			
		return ;
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "carry");
		carry = run_carry = null<number>;
		return ;
	}
}

/*-----------------------------------------------------------------------
  roll_down		EXT SWAP: ok
  ----------------------------------------------------------------------*/
void roll_down(	swap_fixflt sw,
				date 		fwd_settle_date,				
				out number  roll,
				out number  run_roll,				
				error_info option(nullable) error )
{
	
	try {
		error_info ee 				= new error_info(true,true);
		date fwd_trade_date 		= null<date>;		
		logical incl_reinv_in_roll 	= false;
		logical use_ai_adj 			= true;

		number roll_upfront, roll_fwd,roll_upfront_ai_adj,roll_fwd_ai_adj;
		number npv_spot_fix_leg, npv_spot_flt_leg, npv_fwd_fix_leg, npv_fwd_flt_leg;
		number reinv_irim_cf_fix_leg, reinv_irim_cf_flt_leg, irim_cf_fix_leg, irim_cf_flt_leg;
		number npv_spot_ir_fix_leg, npv_spot_ir_flt_leg;
		
		sw.upfront_rolldown_all(fwd_trade_date,fwd_settle_date,incl_reinv_in_roll,
								roll_upfront,roll_fwd,roll_upfront_ai_adj,roll_fwd_ai_adj,npv_spot_fix_leg,npv_spot_flt_leg,
								npv_fwd_fix_leg,npv_fwd_flt_leg,reinv_irim_cf_fix_leg,
								reinv_irim_cf_flt_leg,irim_cf_fix_leg,irim_cf_flt_leg,
								npv_spot_ir_fix_leg, npv_spot_ir_flt_leg,ee);

		
		number sw_rate 			= sw.coupon_fix_leg(ee);
		disc_func df			= sw.disc_func_fix_leg(ee);
		tenor_surface ts 		= sw.tenor_surface_flt_leg(ee);
		tenor_code tc			= sw.index_tenor_code_flt_leg(ee) ;
		fwd_func ff				= ts.fwd_curve_f(tc);
		date start_date 		= use_ai_adj ? fwd_settle_date : sw.start_date();
		
		swap_fixflt sw_fwd		= swap_fixflt_plain(sw, "fwd_stub", fwd_trade_date,fwd_settle_date,start_date,
													null<string>,sw.maturity(),null<string>, sw_rate ,									
													sw.notional(), sw.pay_fix(),df,ff, null<number>,-1,ee);		
		
		number _pv01			= sw_fwd.pv01(null<number>,null<number>, null<disc_func>,ee);
		roll 					= use_ai_adj ? roll_fwd_ai_adj : roll_fwd;
		run_roll 				= roll / _pv01;
			
		return ;
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "roll");
		roll = run_roll = null<number>;
		return ;
	}
}


/*void swap_fixflt.upfront_carryroll(out date   fwd_trade_date,
									out date  	fwd_settle_date,
									out number 	carry_roll,
									out number 	carry_roll_ai_adj,									
									error_info option(nullable) error )*/

/*-----------------------------------------------------------------------
  carry_roll_down		EXT SWAP: ok
  ----------------------------------------------------------------------*/
void carry_roll_down(	swap_fixflt sw,
						date 		fwd_settle_date,
						out number  carry_roll,
						out number  run_carry_roll,
						out number  carry,
						out number  run_carry,
						out number  carry_imp,
						out number  run_carry_imp,
						out number  roll,
						out number  run_roll,
						out logical ai_adjusted,
						error_info option(nullable) error )
{
	
	try {
		error_info ee 			= new error_info(true,true);
		date fwd_trade_date 	= null<date>;
		
		sw.upfront_carryroll(fwd_settle_date,carry_roll,roll,carry_imp,ai_adjusted,ee );
		//roll --> roll_fwd_ai_adj
		..carry(sw,fwd_settle_date,carry,run_carry,ee );
		
		number sw_rate 		= sw.coupon_fix_leg(ee);
		disc_func df		= sw.disc_func_fix_leg(ee);
		tenor_surface ts 	= sw.tenor_surface_flt_leg(ee);
		tenor_code tc		= sw.index_tenor_code_flt_leg(ee) ;
		fwd_func ff			= ts.fwd_curve_f(tc);
		
		swap_fixflt sw_fwd	= swap_fixflt_plain(sw, "fwd_stub", fwd_trade_date,fwd_settle_date,fwd_settle_date,
													null<string>,sw.maturity(),null<string>, sw_rate ,									
													sw.notional(), sw.pay_fix(),df,ff, null<number>,-1,ee);		
		
		number _pv01		= sw_fwd.pv01(null<number>,null<number>, null<disc_func>,ee);
		run_roll 			= roll / _pv01;
		run_carry_roll		= carry_roll / _pv01;
		run_carry_imp		= carry_imp/ _pv01;
		
		return ;
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "roll");
		carry_roll = run_carry_roll = carry = run_carry = roll = run_roll = carry_imp = run_carry_imp = null<number>;
		return ;
	}
}

/*-----------------------------------------------------------------------
  carry_roll_down		EXT SWAP: ok
  ----------------------------------------------------------------------*/
void carry_roll_down(	swap_fixflt sw_pay,
						swap_fixflt sw_rec,
						date 		fwd_settle_date,
						logical 	use_pv01_pay,
						out number  carry_roll,
						out number  run_carry_roll,
						out number  carry,
						out number  run_carry,
						out number  carry_imp,
						out number  run_carry_imp,
						out number  roll,
						out number  run_roll,
						out logical ai_adjusted,
						error_info option(nullable) error )
{
	
	try {
		error_info ee 				= new error_info(true,true);
		date fwd_trade_date 		= null<date>;

		QL_REQUIRE(sw_pay.notional() == sw_rec.notional(),"notional must be the same in both swaps");
		
		number carry_rf_pay,roll_f_pay,carry_imp_f_pay;
		logical ai_adj_pay;
		sw_pay.upfront_carryroll(fwd_settle_date,carry_rf_pay,roll_f_pay,carry_imp_f_pay,ai_adj_pay,ee );
		//roll --> roll_fwd_ai_adj
		number carry_f_pay,run_carry_f_pay;
		..carry(sw_pay,fwd_settle_date,carry_f_pay,run_carry_f_pay,ee );

		number carry_rf_rec,roll_f_rec,carry_imp_f_rec;
		logical ai_adj_rec;		
		sw_rec.upfront_carryroll(fwd_settle_date,carry_rf_rec,roll_f_rec,carry_imp_f_rec,ai_adj_rec,ee );
		//roll --> roll_fwd_ai_adj
		number carry_f_rec,run_carry_f_rec;
		..carry(sw_rec,fwd_settle_date,carry_f_rec,run_carry_f_rec,ee );

		carry_roll			= carry_rf_pay + carry_rf_rec;
		carry				= carry_f_pay + carry_f_rec;
		carry_imp			= carry_imp_f_pay+carry_imp_f_rec;
		roll				= roll_f_pay+roll_f_rec;

		swap_fixflt	sw_risk	= use_pv01_pay ? sw_pay:sw_rec;
		
		number sw_rate 		= sw_risk.coupon_fix_leg(ee);
		disc_func df		= sw_risk.disc_func_fix_leg(ee);
		tenor_surface ts 	= sw_risk.tenor_surface_flt_leg(ee);
		tenor_code tc		= sw_risk.index_tenor_code_flt_leg(ee) ;
		fwd_func ff			= ts.fwd_curve_f(tc);
		
		swap_fixflt sw_fwd	= swap_fixflt_plain(sw_risk, "fwd_stub", fwd_trade_date,fwd_settle_date,fwd_settle_date,
													null<string>,sw_risk.maturity(),null<string>, sw_rate ,									
													sw_risk.notional(), sw_risk.pay_fix(),df,ff, null<number>,-1,ee);		
		
		number _pv01		= sw_fwd.pv01(null<number>,null<number>, null<disc_func>,ee);

		run_carry			= carry / _pv01;
		run_roll 			= roll / _pv01;
		run_carry_roll		= carry_roll / _pv01;
		run_carry_imp		= carry_imp/ _pv01;
		
		return ;
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "roll");
		carry_roll = run_carry_roll = carry = run_carry = roll = run_roll = carry_imp = run_carry_imp = null<number>;
		return ;
	}
}