option(null: hard);	

/*	
	instrument wrapper functions
	Developer: Algorithmica Research, Magnus Nyström

	--------------------------------------------------------------------
	 	swap_fixois_asw create funcs
	--------------------------------------------------------------------			
*/

//--------
// from db  --> NOT POSSIBLE
//--------


/*-----------------------------------------------------------------------
  func: swap_fixois_bond_match  
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_bond_match(	string 							name,
									bond							b,
									INSTR_TMPL.swap_fixois_def_tmpl	tmpl,
									number option(nullable)			nominal,
									disc_func 						disc_func,									
									//fwd_func  						flt_fwd_func,	
									//number option(nullable) 		curr_fixing,									
									out number 	spread,
									error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs Float')
{	
	try{
		CORE_INT.reset_single(error);
		error_info ee 				= error_info(true,true);
		instr_def def 				= instr_def_swap_fixois(tmpl,ee);

		logical is_ccy,fix_end_of_month,ois_end_of_month;
		string fix_ccy, ois_ccy ;
		day_count_method fix_dc, ois_dc;
		notional_exchg_style ne;
		date_code settle_code;
		coupon_freq  fix_cpn_freq, ois_cpn_freq;
		
		calendar fix_pmt_calendar,ois_pmt_calendar;
		bd_convention fix_pmt_bus_day,ois_pmt_bus_day;
		reset_freq  ois_r_freq;
		ir_index ois_ir_index;
		flt_comp_avg_type ois_ca_t;
		flt_sprd_comp_method ois_comp_m;
		flt_avg_method 	ois_avg;

		round_code 	ois_comp_factor_rc ;
		round_code  ois_imp_rate_pcnt_rc;							
		interest_rule  ois_ir_rule ;															
		number  ois_pmt_delay 	;									
		rfr_arrears_type  ois_arr_type 	;
		number  ois_arr_days 	;
		logical ois_endog_df ;
		
		CORE_INT_FIXOIS.unpack_instr_def(def,is_ccy,ne, settle_code,fix_cpn_freq,fix_ccy,fix_pmt_calendar, fix_pmt_bus_day, fix_end_of_month, fix_dc,														
										ois_cpn_freq, ois_ccy, ois_pmt_calendar, ois_pmt_bus_day, ois_end_of_month, ois_dc,
										ois_r_freq, ois_ir_index, ois_ca_t, ois_comp_m, ois_avg,ois_comp_factor_rc, ois_imp_rate_pcnt_rc,
										ois_ir_rule,ois_pmt_delay, ois_arr_type, ois_arr_days, ois_endog_df);
		
		QL_FAIL_COND(is_ccy, "constructor not applicable for currency swap", E_INIT);

		integer 	bfreq	= b.coupon_freq(ee);		
		fix_cpn_freq 		= coupon_freq_from_int(bfreq)  ;
		
		ql_swap_fix_def fix_def 	= ql_swap_fix_def("fix_def", fix_ccy, settle_code, fix_cpn_freq, fix_dc,fix_pmt_bus_day, fix_pmt_calendar,fix_end_of_month,ne);

		logical average_notional = null<logical>;
		number sub_factor 		= null<number>;
		number multi_factor 	= null<number>;
		number power_factor 	= null<number>;	
		
		ql_swap_ois_def ois_def 	= ql_swap_ois_def(	"ois_def",ois_ccy,settle_code, ois_cpn_freq,ois_dc, flt_roll_method.FLR_STRAIGHT,ois_pmt_bus_day,	
														ois_pmt_calendar,ois_end_of_month,ois_ir_index,null<quote_style>,null<round_code>,
														ois_comp_factor_rc,ois_imp_rate_pcnt_rc,ois_ir_rule, ois_pmt_delay, ois_pmt_delay,
														ois_pmt_delay, ois_pmt_delay,ois_arr_type,ois_arr_days,ois_ca_t,ois_comp_m,ois_avg, ne,
														average_notional,sub_factor,multi_factor,power_factor,ois_endog_df);
		QL_FAIL_COND(is_ccy,"inapplicable function call for cross currency swaps");
		string curr 				= b.currency(ee);
		
		QL_FAIL_COND(!equal_casei(curr,fix_ccy),"bond currency does not match currency of swap template");

		date trade_date				= b.trade_date(ee);
		date settle_date 			= b.settle_date(ee);
		date maturity 				= b.maturity(ee);		
		
		..disc_func fix_disc_func, ois_disc_func;
		fwd_func ois_fwd_func;
		
		CORE_INT_FIXOIS.ois_df_fwd_helper(is_ccy,disc_func,ois_fwd_func,disc_func,ois_ir_index,ois_endog_df,trade_date,
										settle_date,maturity,"",fix_disc_func,ois_disc_func);


		date  fix_first_cpn_date 	= b.next_cpn_date(ee);
		date  fix_last_reg_date 	= b.last_reg_cpn_date(ee);
		date  ois_first_cpn_date 	= null<date>;
		date  ois_last_reg_date 	= null<date>;					
		date start_date 			= settle_date;
		logical pay_fixed			= true;
		number cpn_roll_day 		= 0;
		if(null(nominal)) nominal 	= b.nominal(ee);
		
		ql_fix_leg fix 		= ql_fix_leg(fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,null,pay_fixed,
										nominal,start_date,fix_first_cpn_date,fix_last_reg_date,0,	fix_disc_func,cpn_roll_day,1);

		number fix_proxy ;
		vector(date) fixing_dates ;	
		vector(number) fixing_rates;
		logical  allow_fwd_fix;

		logical ois_approx 		= true;
		ql_ois_leg ois 			= ql_ois_leg(ois_def,"ois_leg",trade_date,settle_date,start_date,maturity,null,
											!pay_fixed,nominal,start_date,ois_first_cpn_date,ois_last_reg_date,0,ois_disc_func,	
											ois_fwd_func,cpn_roll_day,1,
											fixing_dates,fixing_rates,fix_proxy,allow_fwd_fix, ois_approx);
		
		ql_fixed_income_swap sw = ql_fixed_income_swap(fix,ois,"temp");

	
		number fix_r = sw.solver(0,SS_FIX_RATE_LEG1,false);
		QL_FAIL_COND(null(fix_r),"error solving for fixed coupon rate");

		fix 	= ql_fix_leg(	fix_def,"fix_leg",trade_date,settle_date,start_date,maturity,null,pay_fixed,
								nominal,start_date,fix_first_cpn_date,fix_last_reg_date,fix_r,fix_disc_func,cpn_roll_day,1);
		
		sw 		= ql_fixed_income_swap(fix,ois,name);
		
		spread 	= b.yield(false,ee)-fix_r;

		return swap_fixois(sw, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fixois_bond_match");
		return null<swap_fixois>;
	}
}