option(null: hard);	

/*	
	instrument wrapper functions
	Developer: Algorithmica Research, Magnus Nyström

	--------------------------------------------------------------------
		swap_fltflt create funcs
	--------------------------------------------------------------------			
*/

//--------
// from db
//--------

/*-----------------------------------------------------------------------
  swap_fltflt
  note: this swap will have same notional and fx=1 on both legs even if cross currency (use add functions to update the swap)
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt(instrument_name 			instrument_name, 
							date option(nullable) 	trade_date 	= null<date>,
							string option(nullable) quote_side 	= null<string>,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{	
	try{			
		__instrument c = __instrument(instrument_name,trade_date,quote_side, null<date> );		
		return create_swap_fltflt(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt");
		return null<swap_fltflt>;
	}
}


/*-----------------------------------------------------------------------
  swap_fltflt
  note: this swap will have same notional and fx=1 on both legs even if cross currency (use add functions to update the swap)
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt(instrument_name 			instrument_name, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,//spread on leg1
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{	
	try{				
		__instrument c = __instrument(instrument_name,trade_date,quote, null<date> );
		return create_swap_fltflt(c,error, E_INIT);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt");
		return null<swap_fltflt>;
	}
}
/*-----------------------------------------------------------------------
  swap_fltflt
  note: this swap will have same notional and fx=1 on both legs even if cross currency (use add functions to update the swap)
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt(instr_def 					instr_def, 
							date option(nullable) 	trade_date 	= null<date>,
							string option(nullable) quote_side 	= null<string>,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{	
	try{				
		__instrument c = __instrument(instr_def,trade_date,quote_side, null<date> );
		return create_swap_fltflt(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt");
		return null<swap_fltflt>;
	}
}
/*-----------------------------------------------------------------------
  swap_fltflt
  note: this swap will have same notional and fx=1 on both legs even if cross currency (use add functions to update the swap)
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt(instr_def 					instr_def, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,//spread on leg1
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{	
	try{			
		__instrument c = __instrument(instr_def,trade_date,quote, null<date> );
		return create_swap_fltflt(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt");
		return null<swap_fltflt>;
	}
}

//----------------
// from db or not
//----------------

/*-----------------------------------------------------------------------
  func: swap_fltflt
  arguments: 
	instr_def: 	can be created either from the db or user defined 				 
				instr_def is copied and modified
				no lookup of quotes
	instrument_name: user defined name
--> will throw for currency swaps
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt(instr_def 				instr_def, 
							string 					instrument_name  ,
							date option(nullable) 	trade_date ,
							date option(nullable) 	settle_date ,
							date  option(nullable) 	maturity,//maturity has precedence over mat_code
							string option(nullable)	maturity_code,
							number option(nullable)	spread,		//spread on leg1				
							number 					nominal = 100,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{	
	try{
		QL_FAIL_COND(instr_def.is_ccy_swap(), "inapplicable function call for cross currency swaps");//because we have only one nominal as input
		__instrument c;
		
		if(!null(maturity))
			c = __instrument_fltflt_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity,spread,nominal );
		else if(!null(maturity_code))
			c = __instrument_fltflt_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity_code,spread,nominal );
		else
			QL_FAIL("invalid maturity/maturity_code");
		return create_swap_fltflt( c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt");
		return null<swap_fltflt>;
	}
}


//----------------
// no db, not par 
//----------------

/*-----------------------------------------------------------------------
  func: swap_fltflt
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt(ql_float_leg 	ql_float_leg1, 							
						ql_float_leg 	ql_float_leg2,
						string 			id,
						error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{	
	try{			
		
		__instrument c = __instrument_swap(ql_float_leg1,ql_float_leg2,id);
		return create_swap_fltflt( c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt");
		return null<swap_fltflt>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fltflt
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt(ql_fixed_income_swap ql_fixed_income_swap,
						error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{	
	try{	
		QL_FAIL_COND(CORE_INT.swap_instr_type(ql_fixed_income_swap) != instr_type.SWAP_FLTFLT, "invalid instrument type");

		__instrument c = __instrument_swap(ql_fixed_income_swap);
		return create_swap_fltflt( c, error);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt");
		return null<swap_fltflt>;
	}
}

module CORE_INT_FLTFLT
{
	/*-----------------------------------------------------------------------
	  swap_lib_fx_handler_ccy
	  ----------------------------------------------------------------------*/
	void swap_lib_fx_handler_ccy(string flt1_curr,
							 string flt2_curr,
							 string base_ccy,
							 string price_ccy,
							 logical pv_in_base_ccy,
							 number fxs_quote_used,
							 out logical leg1_is_base_ccy,
							 out number flt1_fx_rate,
							 out number flt2_fx_rate)
	{
		leg1_is_base_ccy 	= equal_casei(flt1_curr,base_ccy);//ex. true (flip fx: false)
		logical leg2_is_base_ccy = equal_casei(flt2_curr,base_ccy); 
		QL_FAIL_COND(!leg1_is_base_ccy && !equal_casei(flt1_curr,price_ccy), "invalid fx_spot currencies", E_INIT);
		QL_FAIL_COND(!leg2_is_base_ccy && !equal_casei(flt2_curr,price_ccy), "invalid fx_spot currencies", E_INIT);

		if(pv_in_base_ccy)//EUR (flip fx: USD)
			flt1_fx_rate = leg1_is_base_ccy ? 1: 1/fxs_quote_used;//ex. 1  		(flip fx: 1/0.7=1.4)
		else
			flt1_fx_rate = leg1_is_base_ccy ? fxs_quote_used: 1 ;//ex. 1.4 		(flip fx: 1)

		if(pv_in_base_ccy)//EUR (flip fx: USD)
			flt2_fx_rate = leg2_is_base_ccy ? 1: 1/fxs_quote_used;//ex. 1/1.4 	(flip fx: 1)
		else
			flt2_fx_rate = leg2_is_base_ccy ? fxs_quote_used :1 ;//ex. 1 		(flip fx: 0.7)
	}
	/*-----------------------------------------------------------------------
	  swap_lib_fx_handler_leg
	  ----------------------------------------------------------------------*/
	void swap_lib_fx_handler_leg(string flt1_curr,
							 string flt2_curr,
							 string base_ccy,
							 string price_ccy,
							 logical pv_in_leg1,
							 number fxs_quote_used,
							 out logical leg1_is_base_ccy,
							 out number flt1_fx_rate,
							 out number flt2_fx_rate)
	{
		leg1_is_base_ccy 	= equal_casei(flt1_curr,base_ccy);//ex. true (flip fx: false)
		logical pv_in_base_ccy = pv_in_leg1? leg1_is_base_ccy : !leg1_is_base_ccy;
		
		swap_lib_fx_handler_ccy(flt1_curr,flt2_curr,base_ccy,price_ccy,pv_in_base_ccy,
							 fxs_quote_used,leg1_is_base_ccy,flt1_fx_rate,flt2_fx_rate);
		
	}
	
	/*-----------------------------------------------------------------------
	  func: swap_fltflt_plain_int
	  minimal constructor for "screen"-swap  --> NO STUBS
	  normally: leg1 = sprd_leg, leg2 = flat_leg
	  ----------------------------------------------------------------------*/
	swap_fltflt swap_fltflt_plain_int(	instr_def 					def,
										string 						instrument_name,	
										date option(nullable) 		trade_date,	
										date option(nullable) 		start_date,
										string option(nullable) 	fwd_start_code,
										date 	option(nullable)	maturity,
										string 	option(nullable)	maturity_code,
										number option(nullable)		flt1_notional,
										number 	 					flt2_notional,
										number option(nullable)		flt1_spread ,//if null --> will be set to par, spread on leg1
										number option(nullable)		flt2_spread ,//if null --> will be set to 0
										logical 					flt1_pay,										
										fx_spot option(nullable)	fx_sp,
										number option(nullable)		fx_quote,//quoted as fx_spot object
										
										..disc_func option(nullable) 	flt1_disc_func,								
										fwd_func option(nullable) 		flt1_fwd_func,
										..disc_func option(nullable) 	flt1_fwd_disc_func,										
										number option(nullable) 		flt1_curr_fix,
										
										..disc_func option(nullable) 	flt2_disc_func,								
										fwd_func option(nullable) 		flt2_fwd_func,
										..disc_func option(nullable) 	flt2_fwd_disc_func,										
										number option(nullable) 		flt2_curr_fix,
										logical option(nullable)		pv_in_base,
										integer 						flt1_sprd_rnd_dec,
										error_info option(nullable) error )
	{	
		try{
			
			CORE_INT.reset_single(error);
			
			QL_FAIL_COND(!def.is_swap_fltflt(), "instr_def is not a float-float swap", E_INIT);
			
			logical is_ccy 		= def.is_ccy_swap();
			logical is_mtm 		= def.is_swap_fltflt_mtm();
			logical is_mtm_leg1 = def.is_swap_fltflt_mtm_leg1();
			//QL_FAIL_COND(is_mtm_leg1, "currently the mtm leg must be leg2", E_INIT);
			
			error_info ee 		= error_info(true,true);
			string flt1_curr	= def.swap_currency_leg1(ee);
			string flt2_curr	= def.swap_currency_leg2(ee);
			notional_exchg_style ne = def.swap_not_exchange(ee);
						
			number flt1_fx_rate ,flt2_fx_rate;
			number fxs_quote_used;
			logical leg1_is_base_ccy;
			date fx_spot_date ;
			
			if(is_ccy) {
				QL_FAIL_COND(null(fx_sp), "fx spot instrument required for cross currency swap", E_INIT);
				fx_spot_date 		= fx_sp.spot_date(ee);
				string base_ccy 	= fx_sp.base_ccy(ee);
				string price_ccy 	= fx_sp.price_ccy(ee);

				if(null(pv_in_base))
					pv_in_base = true;

				fxs_quote_used 	= null(fx_quote) ? fx_sp.quote(true,false,ee) : fx_quote/fx_sp.scale(ee);
				QL_FAIL_COND(null(fxs_quote_used), "invalid fx_spot quote", E_INIT);
				
				swap_lib_fx_handler_ccy(flt1_curr,flt2_curr,base_ccy,price_ccy,pv_in_base,fxs_quote_used,
										leg1_is_base_ccy, flt1_fx_rate, flt2_fx_rate);
				
					
				if(null(ne))
					ne 	= NE_BOTH;	
			}
			else {
				fxs_quote_used = flt1_fx_rate = flt2_fx_rate = 1;
				if(null(ne))
					ne 	= NE_NONE;	
			}
						
			//common
			
			date_code settle_code 			= date_code(def.settle_code());
			flt_reset_style reset_style 	= RS_NORM;

			//leg1
			calendar flt1_pmt_calendar		= def.swap_calendar_leg1(ee);
			bd_convention flt1_pmt_bus_day 	= def.swap_pmt_bus_day_leg1(ee);
			logical	flt1_end_of_month		= def.swap_eom_leg1(ee);
			day_count_method flt1_dc 		= def.swap_dc_method_leg1(ee);			
			coupon_freq  flt1_cpn_freq 		= coupon_freq_from_int(def.swap_coupon_freq_leg1(ee))  ;
			reset_freq  flt1_r_freq 		= reset_freq_from_int(def.swap_reset_freq_leg1(ee))  ;
			ir_index flt1_ir_index 			= def.swap_ir_index_leg1(ee);				
			ql_swap_float_def flt1_def 		= ql_swap_float_def("float_def1",flt1_curr,settle_code,
																flt1_cpn_freq,flt1_r_freq,flt1_dc,flt1_pmt_bus_day,
																flt1_pmt_calendar,flt1_end_of_month,
																null<flt_comp_avg_type>,null<flt_sprd_comp_method>,null<flt_avg_method>,
																flt1_ir_index,reset_style,ne);

			//leg2
			calendar flt2_pmt_calendar		= def.swap_calendar_leg2(ee);
			bd_convention flt2_pmt_bus_day 	= def.swap_pmt_bus_day_leg2(ee);
			logical	flt2_end_of_month		= def.swap_eom_leg2(ee);
			day_count_method flt2_dc 		= def.swap_dc_method_leg2(ee);			
			coupon_freq  flt2_cpn_freq 		= coupon_freq_from_int(def.swap_coupon_freq_leg2(ee))  ;
			reset_freq  flt2_r_freq 		= reset_freq_from_int(def.swap_reset_freq_leg2(ee))  ;
			ir_index flt2_ir_index 			= def.swap_ir_index_leg2(ee);				
			ql_swap_float_def flt2_def 		= ql_swap_float_def("float_def2",flt2_curr,settle_code,
															flt2_cpn_freq,flt2_r_freq,flt2_dc,flt2_pmt_bus_day,
															flt1_pmt_calendar,flt2_end_of_month,
															null<flt_comp_avg_type>,null<flt_sprd_comp_method>,null<flt_avg_method>,
															flt2_ir_index,reset_style,ne);
			
			//---leg---
			//common
			date  first_cpn_date 			= null<date>;
			date  last_reg_date 			= null<date>;
			vector(date) fixing_dates 		= null<vector(date)>;	
			vector(number) fixing_rates		= null<vector(number)>;
			logical  allow_fwd_fix			= false;
			integer	cpn_roll_day 			= 0;			
			calendar cal 					= calendar(flt1_pmt_calendar,flt2_pmt_calendar);
			date settle_date;
			CORE_INT_FIXFLT.swap_start_date_helper(settle_code,trade_date,cal,fwd_start_code,start_date,settle_date);
			QL_FAIL_COND(null(settle_date),"invalid start date", E_INIT);
			tenor_code mat_code 			= CORE_INT_FIXFLT.swap_maturity_helper(def,maturity, maturity_code);
			
			if(is_ccy){				
				QL_FAIL_COND(null(fx_quote) && settle_date != fx_spot_date,"explicit fx quote required when fx spot date is not equal to settle date", E_INIT);
				
				if(null(flt1_notional)){
					QL_FAIL_COND(start_date != settle_date,"invalid nominal (both nominals must be input when settle date is not equal to start date)", E_INIT);					
				}		
			}

			logical allow_extrap 	= false;
			
			//leg1
			number flt1_fix_proxy 	= flt1_curr_fix;			
			date flt1_eff_date		= start_date;			
			number init_sprd 		= null(flt1_spread) ? 0 : flt1_spread;

			if(null(flt1_notional) ) {
				if(!is_ccy)
					flt1_notional = flt2_notional;
				else
					flt1_notional = flt2_notional * flt2_fx_rate / flt1_fx_rate;
			}
			
			tenor_surface flt1_tenor_surface;
			try{
				if(!null(flt1_fwd_func))
					flt1_tenor_surface = tenor_surface(trade_date, flt1_disc_func,[flt1_fwd_func],[flt1_ir_index],allow_extrap);
				else
					flt1_tenor_surface = tenor_surface(trade_date, flt1_disc_func,[flt1_fwd_disc_func],[flt1_ir_index],allow_extrap);
			}
			catch {
				flt1_tenor_surface = null<tenor_surface>;
			}
			
			//leg2
			number flt2_fix_proxy 	= flt2_curr_fix;											
			date flt2_eff_date		= start_date;

			if(null(flt2_spread))
				flt2_spread = 0;
			
			
			tenor_surface flt2_tenor_surface;
			try{
				if(!null(flt2_fwd_func))
					flt2_tenor_surface = tenor_surface(trade_date, flt2_disc_func,[flt2_fwd_func],[flt2_ir_index],allow_extrap);
				else
					flt2_tenor_surface = tenor_surface(trade_date, flt2_disc_func,[flt2_fwd_disc_func],[flt2_ir_index],allow_extrap);
			}
			catch {
				flt2_tenor_surface = null<tenor_surface>;
			}

			
			if(!is_mtm) {
				ql_float_leg flt2 	= ql_float_leg(	flt2_def,"float_leg2",trade_date,settle_date,start_date,maturity,mat_code,
										!flt1_pay,flt2_notional,flt2_eff_date,first_cpn_date,last_reg_date,0,flt2_disc_func,	
										flt2_tenor_surface,cpn_roll_day,flt2_fx_rate,
										fixing_dates,fixing_rates,flt2_fix_proxy,allow_fwd_fix);

				ql_float_leg flt1 	= ql_float_leg(flt1_def,"float_leg1",trade_date,settle_date,start_date,maturity,mat_code,
													flt1_pay,flt1_notional,flt1_eff_date,first_cpn_date,last_reg_date,init_sprd,flt1_disc_func,	
													flt1_tenor_surface,cpn_roll_day,flt1_fx_rate,
													fixing_dates,fixing_rates,flt1_fix_proxy,allow_fwd_fix);

				ql_fixed_income_swap sw = ql_fixed_income_swap(flt1,flt2,instrument_name);

				if(null(flt1_spread)) {
					number flt_s	= sw.solver(0,swap_solver_code.SS_FLT_SPRD_LEG1,false);
					QL_FAIL_COND(null(flt_s),"error solving for spread", E_INIT);
					if(flt1_sprd_rnd_dec >= 0)
						flt_s = round(flt_s,flt1_sprd_rnd_dec);
					flt1 	= ql_float_leg(flt1_def,"float_leg1",trade_date,settle_date,start_date,maturity,mat_code,
											flt1_pay,flt1_notional,flt1_eff_date,first_cpn_date,last_reg_date,flt_s,flt1_disc_func,	
											flt1_tenor_surface,cpn_roll_day,flt1_fx_rate,
											fixing_dates,fixing_rates,flt1_fix_proxy,allow_fwd_fix);
					
					sw 		= ql_fixed_income_swap(flt1,flt2,instrument_name);

				}
				
				return swap_fltflt(sw, error);
			}


			//mtm only from here
			ql_fixed_income_swap sw;
			
			if(!is_mtm_leg1){
			
				date flt_mtm_fx_spot_date 			= fx_spot_date; 
				number flt_mtm_fx_spot_LG2LG1		= flt2_fx_rate/flt1_fx_rate;//the fxrate that makes flatleg(LG2) equal to sprdleg(LG1) i.e. flatlegccy is baseccy
				disc_func flt_mtm_fx_df_LG2			= flt2_disc_func;//leg2 is flat leg and this disc_func is for the leg itself
				disc_func flt_mtm_fx_df_LG1			= flt1_disc_func;//disc_func for "other leg" = leg1 
				number flt_mtm_fx_not_LG1 			= flt1_notional;
				vector(date) flt_mtm_fx_fix_dates 	= null<vector(date)>;
				vector(number) flt_mtm_fx_fix 		= null<vector(number)>;
				number flt_mtm_fx_fix_proxy 		= flt_mtm_fx_spot_LG2LG1;
				logical flt_mtm_fx_fix_as_fwd 		= false;

				 ql_float_leg flt2 	= ql_float_leg(flt2_def,"float_leg2",trade_date,settle_date,start_date,maturity,mat_code,
										!flt1_pay,flt2_notional,flt2_eff_date,first_cpn_date,last_reg_date,flt2_spread,flt2_disc_func,
										flt2_tenor_surface,cpn_roll_day,flt2_fx_rate,
										fixing_dates,fixing_rates,flt2_fix_proxy,allow_fwd_fix,
										flt_mtm_fx_spot_date, flt_mtm_fx_spot_LG2LG1,flt_mtm_fx_df_LG2,flt_mtm_fx_df_LG1,
										flt_mtm_fx_not_LG1,flt_mtm_fx_fix_dates,flt_mtm_fx_fix,flt_mtm_fx_fix_proxy,flt_mtm_fx_fix_as_fwd);

				 ql_float_leg flt1 	= ql_float_leg(flt1_def,"float_leg1",trade_date,settle_date,start_date,maturity,mat_code,
													flt1_pay,flt1_notional,flt1_eff_date,first_cpn_date,last_reg_date,init_sprd,flt1_disc_func,	
													flt1_tenor_surface,cpn_roll_day,flt1_fx_rate,
													fixing_dates,fixing_rates,flt1_fix_proxy,allow_fwd_fix);

				sw = ql_fixed_income_swap(flt1,flt2,instrument_name);

				if(null(flt1_spread)) {
					number flt_s	= sw.solver(0,swap_solver_code.SS_FLT_SPRD_LEG1,false);
					QL_FAIL_COND(null(flt_s),"error solving for spread", E_INIT);
					if(flt1_sprd_rnd_dec >= 0)
						flt_s = round(flt_s,flt1_sprd_rnd_dec);
					flt1 	= ql_float_leg(flt1_def,"float_leg1",trade_date,settle_date,start_date,maturity,mat_code,
											flt1_pay,flt1_notional,flt1_eff_date,first_cpn_date,last_reg_date,flt_s,flt1_disc_func,	
											flt1_tenor_surface,cpn_roll_day,flt1_fx_rate,
											fixing_dates,fixing_rates,flt1_fix_proxy,allow_fwd_fix);
					
					sw 		= ql_fixed_income_swap(flt1,flt2,instrument_name);

				}
			}
			else {

				ql_float_leg flt2 	= ql_float_leg(	flt2_def,"float_leg2",trade_date,settle_date,start_date,maturity,mat_code,
										!flt1_pay,flt2_notional,flt2_eff_date,first_cpn_date,last_reg_date,flt2_spread,flt2_disc_func,	
										flt2_tenor_surface,cpn_roll_day,flt2_fx_rate,
										fixing_dates,fixing_rates,flt2_fix_proxy,allow_fwd_fix);



				date flt_mtm_fx_spot_date 			= fx_spot_date; 
				number flt_mtm_fx_spot_LG1LG2		= flt1_fx_rate/flt2_fx_rate;//the fxrate that makes flatleg(LG2) equal to sprdleg(LG1) i.e. flatlegccy is baseccy
				disc_func flt_mtm_fx_df_LG1			= flt1_disc_func;//leg2 is flat leg and this disc_func is for the leg itself
				disc_func flt_mtm_fx_df_LG2			= flt2_disc_func;//disc_func for "other leg" = leg1 
				number flt_mtm_fx_not_LG2 			= flt2_notional;
				vector(date) flt_mtm_fx_fix_dates 	= null<vector(date)>;
				vector(number) flt_mtm_fx_fix 		= null<vector(number)>;
				number flt_mtm_fx_fix_proxy 		= flt_mtm_fx_spot_LG1LG2;
				logical flt_mtm_fx_fix_as_fwd 		= false;

				 ql_float_leg flt1 	= ql_float_leg(flt1_def,"float_leg1",trade_date,settle_date,start_date,maturity,mat_code,
										flt1_pay,flt1_notional,flt1_eff_date,first_cpn_date,last_reg_date,init_sprd,flt1_disc_func,
										flt1_tenor_surface,cpn_roll_day,flt1_fx_rate,
										fixing_dates,fixing_rates,flt1_fix_proxy,allow_fwd_fix,
										flt_mtm_fx_spot_date, flt_mtm_fx_spot_LG1LG2,flt_mtm_fx_df_LG1,flt_mtm_fx_df_LG2,
										flt_mtm_fx_not_LG2,flt_mtm_fx_fix_dates,flt_mtm_fx_fix,flt_mtm_fx_fix_proxy,flt_mtm_fx_fix_as_fwd);
				
				sw = ql_fixed_income_swap(flt1,flt2,instrument_name);

				if(null(flt1_spread)) {
					number flt_s	= sw.solver(0,swap_solver_code.SS_OIS_SPRD_LEG1,false);
					QL_FAIL_COND(null(flt_s),"error solving for spread", E_INIT);
					if(flt1_sprd_rnd_dec >= 0)
						flt_s = round(flt_s,flt1_sprd_rnd_dec);
					flt1 	= ql_float_leg(flt1_def,"float_leg1",trade_date,settle_date,start_date,maturity,mat_code,
										flt1_pay,flt1_notional,flt1_eff_date,first_cpn_date,last_reg_date,flt_s,flt1_disc_func,
										flt1_tenor_surface,cpn_roll_day,flt1_fx_rate,
										fixing_dates,fixing_rates,flt1_fix_proxy,allow_fwd_fix,
										flt_mtm_fx_spot_date, flt_mtm_fx_spot_LG1LG2,flt_mtm_fx_df_LG1,flt_mtm_fx_df_LG2,
										flt_mtm_fx_not_LG2,flt_mtm_fx_fix_dates,flt_mtm_fx_fix,flt_mtm_fx_fix_proxy,flt_mtm_fx_fix_as_fwd);
					
					sw 		= ql_fixed_income_swap(flt1,flt2,instrument_name);

				}
					
			}			
			
			return swap_fltflt(sw, error);						
		}
		catch {
			CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt_plain_int");
			return null<swap_fltflt>;
		}
	}
}	
/*-----------------------------------------------------------------------
  func: swap_fltflt_plain
  minimal constructor for single curr "screen"-swap, --> NO STUBS
  this version: fwd_func input
  normally: leg1 = sprd_leg, leg2 = flat_leg
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt_plain(	instr_def 					def,
								string 						instrument_name,	
								date option(nullable) 		trade_date,	
								date option(nullable) 		start_date,			//start_date has precedence over start_date_code
								string option(nullable) 	start_date_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,																	
								number option(nullable)		notional_leg1,
								number 	 					notional_leg2,
								number option(nullable)		spread_leg1 ,		//if null --> will be set to par
								number option(nullable)		spread_leg2 ,//if null --> will be set to 0
								logical 					pay_leg1,
								fx_spot option(nullable)	fx_sp,//OBS! fix this for the case when settle != spot date
								number option(nullable)		fx_quote,
								disc_func option(nullable) 	disc_func_leg1,
								fwd_func option(nullable) 	fwd_func_leg1,	
							    number option(nullable) 	curr_fix_leg1,
								disc_func option(nullable) 	disc_func_leg2,
							    fwd_func option(nullable) 	fwd_func_leg2,	
							    number option(nullable) 	curr_fix_leg2,
								logical option(nullable)	pv_in_base_ccy,
								integer 					sprd_rnd_dec_leg1,
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{
	return CORE_INT_FLTFLT.swap_fltflt_plain_int(	def,instrument_name,trade_date,start_date,start_date_code,maturity,maturity_code,
											notional_leg1,notional_leg2,spread_leg1 , spread_leg2,pay_leg1,fx_sp,fx_quote,
											disc_func_leg1,fwd_func_leg1, null<disc_func>,curr_fix_leg1,
											disc_func_leg2, fwd_func_leg2, null<disc_func>,curr_fix_leg2,
											pv_in_base_ccy,sprd_rnd_dec_leg1,error );
}

/*-----------------------------------------------------------------------
  func: swap_fltflt_plain
  minimal constructor for "screen"-swap, --> NO STUBS
  this version: disc_func input
  normally: leg1 = sprd_leg, leg2 = flat_leg
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt_plain(	instr_def 					def,
							string 						instrument_name,	
							date option(nullable) 		trade_date,	
							date option(nullable) 		start_date,			//start_date has precedence over start_date_code
							string option(nullable) 	start_date_code,
							date option(nullable)		maturity,			//maturity has precedence over mat_code
							string 	option(nullable)	maturity_code,																																											
							number option(nullable)		notional_leg1,
							number 	 					notional_leg2,
							number option(nullable)		spread_leg1 ,//if null --> will be set to par
							number option(nullable)		spread_leg2 ,//if null --> will be set to 0
							logical 					pay_leg1,
							fx_spot option(nullable)	fx_sp,//OBS! fix this for the case when settle != spot date
							number option(nullable)		fx_quote,
							disc_func option(nullable) 	disc_func_leg1,
							disc_func option(nullable) 	fwd_disc_func_leg1,	
							number option(nullable) 	curr_fix_leg1,								
							disc_func option(nullable) 	disc_func_leg2,
							disc_func option(nullable) 	fwd_disc_func_leg2,																																												
							number option(nullable) 	curr_fix_leg2,
							logical option(nullable)	pv_in_base_ccy,
							integer 					sprd_rnd_dec_leg1,
							error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{
	return CORE_INT_FLTFLT.swap_fltflt_plain_int(	def,instrument_name,trade_date,start_date,start_date_code,maturity,maturity_code,
											notional_leg1,notional_leg2,spread_leg1 ,spread_leg2,pay_leg1,fx_sp,fx_quote,
											disc_func_leg1,null<fwd_func>, fwd_disc_func_leg1,curr_fix_leg1,
											disc_func_leg2, null<fwd_func>, fwd_disc_func_leg2,curr_fix_leg2,
											pv_in_base_ccy,sprd_rnd_dec_leg1,error );
}
/*-----------------------------------------------------------------------
  func: swap_fltflt_plain  
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt_plain(	instr_def 					instr_def, 
								string 						instrument_name  ,			
								date  option(nullable) 		start_date,
								date  option(nullable)		maturity,
								integer						cpn_roll_day,

								date  option(nullable)		flt1_eff_date,
					    		date  option(nullable)		flt1_first_cpn_date,
					    		date  option(nullable)		flt1_last_reg_date,
								number						flt1_spread,							
								number 	option(nullable) 	flt1_notional,

								date  option(nullable)		flt2_eff_date,
								date  option(nullable)		flt2_first_cpn_date,
					    		date  option(nullable)		flt2_last_reg_date,
								number						flt2_spread,							
								number 	option(nullable) 	flt2_notional,
		
								date option(nullable) 		trade_date ,
								date option(nullable) 		settle_date ,
								logical 					pay_float1,

								fwd_func option(nullable) 		flt1_fwd_func,	
								disc_func option(nullable) 		flt1_disc_func,						
								number option(nullable) 		flt1_fx_rate,//as of settle						
								vector(date) option(nullable) 	flt1_fixing_dates,	
								vector(number) option(nullable) flt1_fixing_rates,
								//number option(nullable) first_fixing,
							    //number option(nullable) last_fixing,
							    number option(nullable) 		flt1_fix_proxy, 
							    logical  option(nullable) 		flt1_allow_fwd_fix,


								fwd_func option(nullable) 		flt2_fwd_func,	
								disc_func option(nullable) 		flt2_disc_func,						
								number option(nullable) 		flt2_fx_rate,//as of settle						
								vector(date) option(nullable) 	flt2_fixing_dates,	
								vector(number) option(nullable) flt2_fixing_rates,
								//number option(nullable) first_fixing,
							    //number option(nullable) last_fixing,
							    number option(nullable) 		flt2_fix_proxy, 
							    logical  option(nullable) 		flt2_allow_fwd_fix,
								error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{	
	try{
		QL_FAIL_COND(!instr_def.is_swap_fltflt(), "instr_def not a float-float swap", E_INIT);
		QL_FAIL_COND(instr_def.is_swap_fltflt_mtm(), "inapplicable function call for mtm-swap", E_INIT);
		//---def---
		logical is_ccy				= instr_def.is_ccy_swap();
		
		error_info ee = error_info(true,true);
		notional_exchg_style ne = instr_def.swap_not_exchange(ee);

		if(null(ne))
			ne 	= is_ccy ? NE_BOTH : NE_NONE;
		
		date_code settle_code 		= date_code(instr_def.settle_code());
		flt_reset_style reset_style = RS_NORM;
				
		calendar pmt_calendar		= instr_def.swap_calendar_leg1(ee);
		bd_convention pmt_bus_day 	= instr_def.swap_pmt_bus_day_leg1(ee);
		logical	end_of_month		= instr_def.swap_eom_leg1(ee);

		coupon_freq  flt1_cpn_freq 	= coupon_freq_from_int(instr_def.swap_coupon_freq_leg1(ee))  ;

		reset_freq	flt1_reset_freq;
		if(instr_def.swap_reset_freq_leg1(ee) == err_freq())
			flt1_reset_freq	= null<reset_freq>  ;
		else
			flt1_reset_freq	= reset_freq_from_int(instr_def.swap_reset_freq_leg1(ee))  ;

		flt_comp_avg_type flt1_comp 		= instr_def.swap_comp_avg_type_leg1(ee);
		flt_sprd_comp_method flt1_comp_m 	= instr_def.swap_sprd_comp_method_leg1(ee);
		flt_avg_method  flt1_avg 			= instr_def.swap_avg_method_leg1(ee);

		ir_index flt1_ir_index 		= instr_def.swap_ir_index_leg1(ee);
				
		ql_swap_float_def flt1_def 	= ql_swap_float_def("float_def1",instr_def.swap_currency_leg1(ee),settle_code,
														flt1_cpn_freq,flt1_reset_freq,instr_def.swap_dc_method_leg1(ee),pmt_bus_day,
														pmt_calendar,end_of_month,
														flt1_comp,flt1_comp_m,flt1_avg,
														flt1_ir_index,reset_style,ne);
		
		
		coupon_freq  flt2_cpn_freq 	= coupon_freq_from_int(instr_def.swap_coupon_freq_leg2(ee))  ;
		reset_freq	flt2_reset_freq;
		if(instr_def.swap_reset_freq_leg2(ee) == err_freq())
			flt2_reset_freq	= null<reset_freq>  ;
		else
			flt2_reset_freq	= reset_freq_from_int(instr_def.swap_reset_freq_leg2(ee))  ;

		flt_comp_avg_type flt2_comp 		= instr_def.swap_comp_avg_type_leg2(ee);
		flt_sprd_comp_method flt2_comp_m 	= instr_def.swap_sprd_comp_method_leg2(ee);
		flt_avg_method  flt2_avg 			= instr_def.swap_avg_method_leg2(ee);

		ir_index flt2_ir_index 		= instr_def.swap_ir_index_leg2(ee);
				
		ql_swap_float_def flt2_def 	= ql_swap_float_def("float_def2",instr_def.swap_currency_leg2(ee),settle_code,
														flt2_cpn_freq,flt2_reset_freq,instr_def.swap_dc_method_leg2(ee),pmt_bus_day,
														pmt_calendar,end_of_month,flt2_comp,flt2_comp_m,flt2_avg,
														flt2_ir_index,reset_style,ne);

		//---leg---
		//date  flt1_first_cpn_date = null<date>;
		//date  flt1_last_reg_date = null<date>;
		//date  flt2_first_cpn_date = null<date>;
		//date  flt2_last_reg_date = null<date>;
		if(null(start_date)) {
			if(!null(instr_def.fwd_start_code(ee))){
				date_code start_code = date_code(instr_def.fwd_start_code(ee));
				start_date = start_code.apply_fwd(trade_date,pmt_calendar);
				start_date	= pmt_calendar.adjust_date (start_date, bd_convention.BD_FOLLOWING);
			}
			else if(!null(settle_date)){
				start_date = settle_date;
			}
			else if(!null(trade_date)){
				settle_date = settle_code.apply_fwd(trade_date,pmt_calendar);
				start_date = settle_date;
			}
		}

		tenor_code mat_code;
		if(null(maturity)) {
			string mc = instr_def.maturity_code(ee);
			if(!null(mc))
				mat_code = tenor_code(mc);
			else
				QL_FAIL("maturity code not defined in instr_def");
		}

		logical allow_extrap 			= false;
		tenor_surface flt1_tenor_surface;
		
		try {
			flt1_tenor_surface = tenor_surface(trade_date, flt1_disc_func,[flt1_fwd_func],[flt1_ir_index],allow_extrap);
		}
		catch {
			flt1_tenor_surface = null<tenor_surface>;
		}
	
		if(null(flt1_fx_rate)) 
			flt1_fx_rate = 1;
		if(null(flt1_eff_date))
			flt1_eff_date = start_date;
		
		ql_float_leg flt1 	= ql_float_leg(	flt1_def,"float_leg1",trade_date,settle_date,start_date,maturity,mat_code,
											pay_float1,flt1_notional,flt1_eff_date,flt1_first_cpn_date,flt1_last_reg_date,flt1_spread,flt1_disc_func,	
											flt1_tenor_surface,cpn_roll_day,flt1_fx_rate,
											flt1_fixing_dates,flt1_fixing_rates,flt1_fix_proxy,flt1_allow_fwd_fix);
		
		if(null(flt2_notional) && !is_ccy) 
			flt2_notional = flt1_notional;
		if(null(flt2_fx_rate)) 
			flt2_fx_rate = 1;
		if(null(flt2_eff_date))
			flt2_eff_date = start_date;
		
		tenor_surface flt2_tenor_surface;
		
		try {	
			flt2_tenor_surface = tenor_surface(trade_date, flt2_disc_func,[flt2_fwd_func],[flt2_ir_index],allow_extrap);
		}
		catch {
			flt2_tenor_surface = null<tenor_surface>;
		}

		ql_float_leg flt2 	= ql_float_leg(flt2_def,"float_leg2",trade_date,settle_date,start_date,maturity,mat_code,
														!pay_float1,flt2_notional,flt2_eff_date,flt2_first_cpn_date,flt2_last_reg_date,flt2_spread,flt2_disc_func,	
														flt2_tenor_surface,cpn_roll_day,flt2_fx_rate,
														flt2_fixing_dates,flt2_fixing_rates,flt2_fix_proxy,flt2_allow_fwd_fix);
		/*
		 date flt_mtm_fx_spot_date;
		number flt_mtm_fx_spot_rate;
		disc_func flt_mtm_fx_base_df;
		disc_func flt_mtm_fx_price_df;
		number flt_mtm_fx_notional;
		vector(date) flt_mtm_fx_fix_dates = null<vector(date)>;
		vector(number) flt_mtm_fx_fix = null<vector(number)>;
		number flt_mtm_fx_fix_proxy = null<number>;
		logical flt_mtm_fx_fix_as_fwd = false;


		 ql_float_leg flt2 				= ql_float_leg(flt2_def,"float_leg2",trade_date,settle_date,start_date,maturity,null<tenor_code>,
														!pay_float1,flt2_notional,start_date,flt2_first_cpn_date,flt2_last_reg_date,flt2_spread,flt2_disc_func,
														flt2_tenor_surface,cpn_roll_day,flt2_fx_rate,
														flt2_fixing_dates,flt2_fixing_rates,flt2_fix_proxy,flt2_allow_fwd_fix,
														flt_mtm_fx_spot_date, flt_mtm_fx_spot_rate,flt_mtm_fx_base_df,flt_mtm_fx_price_df,
														flt_mtm_fx_notional,flt_mtm_fx_fix_dates,flt_mtm_fx_fix,flt_mtm_fx_fix_proxy,flt_mtm_fx_fix_as_fwd)*/
		ql_fixed_income_swap sw 	= ql_fixed_income_swap(flt1,flt2,instrument_name);

		return swap_fltflt(sw, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt_plain");
		return null<swap_fltflt>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fltflt_plain
  //no mtm
  ----------------------------------------------------------------------*/
swap_fltflt swap_fltflt_plain(	string 						instrument_name  ,			
								date  option(nullable) 		start_date,
								date  						maturity,
								integer						cpn_roll_day,
								bd_convention				pmt_bus_day,
								calendar					pmt_calendar,
								logical						end_of_month,
								date_code					settle_code,
								notional_exchg_style option(nullable)  not_exchange,

								date  option(nullable)		flt1_eff_date,
					    		date  option(nullable)		flt1_first_cpn_date,
					    		date  option(nullable)		flt1_last_reg_date,
								day_count_method			flt1_day_count,
								number						flt1_spread,
								integer 					flt1_coupon_freq ,	
								number option(nullable)		flt1_reset_freq,						
								number 	option(nullable) 	flt1_notional,
								string						flt1_currency,
								flt_comp_avg_type 			flt1_comp ,
								flt_sprd_comp_method 		flt1_comp_m ,
								flt_avg_method  			flt1_avg ,
								ir_index 					flt1_ir_index,
								
								date  option(nullable)		flt2_eff_date,
								date  option(nullable)		flt2_first_cpn_date,
					    		date  option(nullable)		flt2_last_reg_date,
								day_count_method			flt2_day_count,
								number						flt2_spread,
								integer 					flt2_coupon_freq ,	
								number option(nullable)		flt2_reset_freq,						
								number 	option(nullable) 	flt2_notional,
								string						flt2_currency,
								flt_comp_avg_type 			flt2_comp ,
								flt_sprd_comp_method 		flt2_comp_m ,
								flt_avg_method  			flt2_avg ,
								ir_index 					flt2_ir_index,
		
								date option(nullable) 		trade_date ,
								date option(nullable) 		settle_date ,
								logical 					pay_float1,

								fwd_func option(nullable) 		flt1_fwd_func,	
								disc_func option(nullable) 		flt1_disc_func,						
								number option(nullable) 		flt1_fx_rate,//as of settle						
								vector(date) option(nullable) 	flt1_fixing_dates,	
								vector(number) option(nullable) flt1_fixing_rates,
								//number option(nullable) first_fixing,
							    //number option(nullable) last_fixing,
							    number option(nullable) 		flt1_fix_proxy, 
							    logical  option(nullable) 		flt1_allow_fwd_fix,


								fwd_func option(nullable) 		flt2_fwd_func,	
								disc_func option(nullable) 		flt2_disc_func,						
								number option(nullable) 		flt2_fx_rate,//as of settle						
								vector(date) option(nullable) 	flt2_fixing_dates,	
								vector(number) option(nullable) flt2_fixing_rates,
								//number option(nullable) first_fixing,
							    //number option(nullable) last_fixing,
							    number option(nullable) 		flt2_fix_proxy, 
							    logical  option(nullable) 		flt2_allow_fwd_fix,
								error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs Float')
{	
	try{				
		logical is_ccy				= flt1_currency != flt2_currency;

		if(null(not_exchange))
			not_exchange 			= is_ccy ? NE_BOTH : NE_NONE;

		flt_reset_style reset_style = RS_NORM;
				
		coupon_freq  flt1_cpn_freq 	= coupon_freq_from_int(flt1_coupon_freq)  ;

		reset_freq	flt1_r_freq;
		if(null(flt1_reset_freq))
			flt1_r_freq = null<reset_freq>;
		else
			flt1_r_freq = reset_freq_from_int(integer(flt1_reset_freq))  ;
				
		ql_swap_float_def flt1_def 	= ql_swap_float_def("float_def1",flt1_currency,settle_code,
														flt1_cpn_freq,flt1_r_freq,flt1_day_count,pmt_bus_day,
														pmt_calendar,end_of_month,
														flt1_comp,flt1_comp_m,flt1_avg,
														flt1_ir_index,reset_style,not_exchange);
		
		
		coupon_freq  flt2_cpn_freq 	= coupon_freq_from_int(flt2_coupon_freq)  ;
		reset_freq	flt2_r_freq;
		if(null(flt2_reset_freq))
			flt2_r_freq = null<reset_freq>;
		else
			flt2_r_freq = reset_freq_from_int(integer(flt2_reset_freq))  ;
	
		ql_swap_float_def flt2_def 	= ql_swap_float_def("float_def2",flt2_currency,settle_code,
														flt2_cpn_freq,flt2_r_freq,flt2_day_count,pmt_bus_day,
														pmt_calendar,end_of_month,
														flt2_comp,flt2_comp_m,flt2_avg,
														flt2_ir_index,reset_style,not_exchange);

		//---leg---
		//date  flt1_first_cpn_date = null<date>;
		//date  flt1_last_reg_date = null<date>;
		//date  flt2_first_cpn_date = null<date>;
		//date  flt2_last_reg_date = null<date>;
		if(null(start_date)) {
			if(!null(settle_date))
				start_date = settle_date;
			else if(!null(trade_date)){
				settle_date = settle_code.apply_fwd(trade_date,pmt_calendar);
				start_date = settle_date;
			}
		}

		logical allow_extrap 			= false;

		tenor_surface flt1_tenor_surface;
		try {
			flt1_tenor_surface = tenor_surface(trade_date, flt1_disc_func,[flt1_fwd_func],[flt1_ir_index],allow_extrap);
		}
		catch {
			flt1_tenor_surface = null<tenor_surface>;
		}
	
		if(null(flt1_fx_rate)) 
			flt1_fx_rate = 1;
		if(null(flt1_eff_date))
			flt1_eff_date = start_date;
		
		ql_float_leg flt1 				= ql_float_leg(flt1_def,"float_leg1",trade_date,settle_date,start_date,maturity,null<tenor_code>,
														pay_float1,flt1_notional,flt1_eff_date,flt1_first_cpn_date,flt1_last_reg_date,flt1_spread,flt1_disc_func,	
														flt1_tenor_surface,cpn_roll_day,flt1_fx_rate,
														flt1_fixing_dates,flt1_fixing_rates,flt1_fix_proxy,flt1_allow_fwd_fix);
		
		if(null(flt2_notional) && !is_ccy) 
			flt2_notional = flt1_notional;
		if(null(flt2_fx_rate)) 
			flt2_fx_rate = 1;
		if(null(flt2_eff_date))
			flt2_eff_date = start_date;
		
		tenor_surface flt2_tenor_surface;
		try {	
			flt2_tenor_surface = tenor_surface(trade_date, flt2_disc_func,[flt2_fwd_func],[flt2_ir_index],allow_extrap);
		}
		catch {
			flt2_tenor_surface = null<tenor_surface>;
		}

		ql_float_leg flt2 				= ql_float_leg(flt2_def,"float_leg2",trade_date,settle_date,start_date,maturity,null<tenor_code>,
														!pay_float1,flt2_notional,flt2_eff_date,flt2_first_cpn_date,flt2_last_reg_date,flt2_spread,flt2_disc_func,	
														flt2_tenor_surface,cpn_roll_day,flt2_fx_rate,
														flt2_fixing_dates,flt2_fixing_rates,flt2_fix_proxy,flt2_allow_fwd_fix);
	
		/*
		 date flt_mtm_fx_spot_date;
		number flt_mtm_fx_spot_rate;
		disc_func flt_mtm_fx_base_df;
		disc_func flt_mtm_fx_price_df;
		number flt_mtm_fx_notional;
		vector(date) flt_mtm_fx_fix_dates = null<vector(date)>;
		vector(number) flt_mtm_fx_fix = null<vector(number)>;
		number flt_mtm_fx_fix_proxy = null<number>;
		logical flt_mtm_fx_fix_as_fwd = false;


		 ql_float_leg flt2 				= ql_float_leg(flt2_def,"float_leg2",trade_date,settle_date,start_date,maturity,null<tenor_code>,
														!pay_float1,flt2_notional,start_date,flt2_first_cpn_date,flt2_last_reg_date,flt2_spread,flt2_disc_func,
														flt2_tenor_surface,cpn_roll_day,flt2_fx_rate,
														flt2_fixing_dates,flt2_fixing_rates,flt2_fix_proxy,flt2_allow_fwd_fix,
														flt_mtm_fx_spot_date, flt_mtm_fx_spot_rate,flt_mtm_fx_base_df,flt_mtm_fx_price_df,
														flt_mtm_fx_notional,flt_mtm_fx_fix_dates,flt_mtm_fx_fix,flt_mtm_fx_fix_proxy,flt_mtm_fx_fix_as_fwd)*/
	
		
		ql_fixed_income_swap sw 		= ql_fixed_income_swap(flt1,flt2,instrument_name);

		return swap_fltflt(sw, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltflt_plain");
		return null<swap_fltflt>;
	}
}



module CORE_INT
{
	/*-----------------------------------------------------------------------
	  func: swap_fltflt_par
	  arguments: 
		instr_class_name
		instrument_name: user defined name
	
	  ----------------------------------------------------------------------*/
	
	/*swap_fltflt swap_fltflt(instr_class_name 	class_name, 
								instrument_name				instrument_name, 									
								date 				trade_date, 
								date				settle_date,
								date 				maturity,
								number				spread,
								out instr_error option(nullable) error)	
	{
		instr_def i_d 	= instr_def(class_name ,true , error);
		if(error.is_error()) return null<swap_fltflt>;
		swap_fltflt sw 	= ..swap_fltflt(	i_d, instrument_name,trade_date,settle_date, 
											maturity,spread,100, error);
		if(error.is_error()) return null<swap_fltflt>;
	}*/

	/*-----------------------------------------------------------------------
	  Name :  	create_par_fltflt
	  Notes :          
	  Changes:
	  			Date    		Who     Detail
	  			?				MN		for backward compat
	  ----------------------------------------------------------------------*/
	swap_fltflt create_par_fltflt(	instr_class_name 	class_name,
									instrument_name		instrument_name, 									
									date 				trade_date, 
									date				settle_date,
									date 				maturity,
									number				spread)
	{
		try{
			error_info ee = new error_info(true,false);
			//swap_fltflt sw = swap_fltflt_par(class_name, instrument_name, trade_date, settle_date, maturity,spread, error);
			instr_def i_d 	= instr_def(class_name ,ee);
			//QL_FAIL_COND(error.is_error(), error.message());
			ee = new error_info(true,true);
			swap_fltflt sw 	= ..swap_fltflt(i_d, instrument_name,trade_date,settle_date, maturity,null<string>,spread,100, ee);			
			
			number q1,q2;
			sw.quote(q1,q2,ee);			
			QL_FAIL_COND(null(q1)|| null(q2),"invalid quote");
			return sw;
		}
		catch {
			CORE_INT.write_warning_message(err.message());
			return null<swap_fltflt>;
		}
	}

	
}
