option(null: hard);	

/*	
	instrument wrapper functions
	Developer: Algorithmica Research, Magnus Nyström	

	--------------------------------------------------------------------
		swap_fltois create funcs - cannot be created from db currently
	--------------------------------------------------------------------		
*/

//--------
// from db
//--------


/*-----------------------------------------------------------------------
  swap_fltois
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois(instrument_name 			instrument_name, 
							date option(nullable) 		trade_date 	= null<date>,
							string option(nullable) 	quote_side 	= null<string>,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{	
	try{			
		__instrument c = __instrument(instrument_name,trade_date,quote_side, null<date> );		
		return create_swap_fltois(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois");
		return null<swap_fltois>;
	}
}


/*-----------------------------------------------------------------------
  swap_fltois
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois(instrument_name 			instrument_name, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,//spread on the ois leg	
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{	
	try{				
		__instrument c = __instrument(instrument_name,trade_date,quote, null<date> );
		return create_swap_fltois(c,error, E_INIT);							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois");
		return null<swap_fltois>;
	}
}


/*-----------------------------------------------------------------------
  swap_fltois
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois(instr_def 					instr_def, 
							date option(nullable) 	trade_date 	= null<date>,
							string option(nullable) quote_side 	= null<string>,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{	
	try{				
		__instrument c = __instrument(instr_def,trade_date,quote_side, null<date> );
		return create_swap_fltois(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois");
		return null<swap_fltois>;
	}
}
/*-----------------------------------------------------------------------
  swap_fltois
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois(instr_def 					instr_def, 
							date option(nullable) 		trade_date ,
							number  option(nullable) 	quote ,//spread on the ois leg	
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{	
	try{			
		__instrument c = __instrument(instr_def,trade_date,quote, null<date> );
		return create_swap_fltois(c,error, E_INIT);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois");
		return null<swap_fltois>;
	}
}

//----------------
// from db or not
//----------------

/*-----------------------------------------------------------------------
  func: swap_fltois
  arguments: 
	instr_def: 	can be created either from the db or user defined 				 
				instr_def is copied and modified
				no lookup of quotes
	instrument_name: user defined name

  ----------------------------------------------------------------------*/
swap_fltois swap_fltois(instr_def 				instr_def, 
							string 					instrument_name  ,
							date option(nullable) 	trade_date ,
							date option(nullable) 	settle_date ,							
							date  option(nullable) 	maturity,//maturity has precedence
							string option(nullable) maturity_code,
							number option(nullable)	spread_vs_ois,	//spread on the ois leg						
							number 					nominal= 100,
							error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{	
	try{
		__instrument c;
		
		if(!null(maturity))
			c = __instrument_fltois_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity,spread_vs_ois,nominal );
		else if(!null(maturity_code))
			c = __instrument_fltois_par_nodb(instr_def,instrument_name,trade_date,settle_date,
														maturity_code,spread_vs_ois,nominal );
		else
			QL_FAIL("invalid maturity/maturity_code");


		return create_swap_fltois( c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois");
		return null<swap_fltois>;
	}
}


//----------------
// no db, not par 
//----------------

/*-----------------------------------------------------------------------
  func: swap_fltois
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois(ql_float_leg 	ql_float_leg,
						ql_ois_leg 		ql_ois_leg, 
						string 			id,
						error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{	
	try{			
		
		__instrument c = __instrument_swap(ql_float_leg,ql_ois_leg,id);

		return create_swap_fltois( c, error);					
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois");
		return null<swap_fltois>;
	}
}

/*-----------------------------------------------------------------------
  func: swap_fltois
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois(ql_fixed_income_swap ql_fixed_income_swap,
						error_info option(nullable) error		= null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{	
	try{	
		QL_FAIL_COND(CORE_INT.swap_instr_type(ql_fixed_income_swap) != instr_type.SWAP_FLTOIS, "invalid instrument type");

		__instrument c = __instrument_swap(ql_fixed_income_swap);

		return create_swap_fltois( c, error);						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois");
		return null<swap_fltois>;
	}
}

module CORE_INT_FLTOIS
{	
	
	/*-----------------------------------------------------------------------
	  Name :  	create_fltois_par
	  Notes :          
	  Changes:
	  			Date    		Who     Detail
	  			?				MN		for backward compat
	  ----------------------------------------------------------------------*/
	swap_fltois create_fltois_par(	instr_class_name 	class_name,
									instrument_name		instrument_name, 									
									date 				trade_date, 
									date				settle_date,
									date 				maturity,
									number				spread_vs_ois)
	{
		try{
			error_info ee = new error_info(true,false);
			instr_def i_d 	= instr_def(class_name , ee);
			
			//QL_FAIL_COND(error.is_error(), error.message());
			ee = new error_info(true,true);
			swap_fltois sw 	= ..swap_fltois(i_d, instrument_name,trade_date,settle_date,
												maturity,null<string>,spread_vs_ois,100, ee);
			//QL_FAIL_COND(null(sw.yield()),"invalid yield");
			return sw;
		}
		catch {
			CORE_INT.write_warning_message(err.message());
			return null<swap_fltois>;
		}
	}
	
	void unpack_instr_def(	instr_def 			def,
							out logical 		is_ccy,
						
							out notional_exchg_style ne,
							out date_code 		settle_code ,	
							out coupon_freq  	ois_cpn_freq ,
							out string 			ois_ccy,
							out calendar 		ois_pmt_calendar,
							out bd_convention 	ois_pmt_bus_day,
							out logical			ois_end_of_month,
							out day_count_method ois_dc,
							out ir_index 		ois_ir_index,
							out flt_comp_avg_type 		ois_ca_t,
							out flt_sprd_comp_method 	ois_comp_m,
							out flt_avg_method 			ois_avg,
							out round_code 			ois_comp_factor_round_code 	,
							out round_code  		ois_imp_rate_pcnt_round_code, 							
							out interest_rule  		ois_ir_rule ,															
							out number  			ois_pmt_delay ,									
							out rfr_arrears_type  	ois_arr_type ,
							out number  			ois_arr_days,
							out logical 			ois_endog_df,
							out flt_reset_style flt_reset_style,
							out coupon_freq  	flt_cpn_freq,
							out string 			flt_ccy,
							out calendar 		flt_pmt_calendar,
							out bd_convention 	flt_pmt_bus_day,
							out logical			flt_end_of_month,
							out day_count_method flt_dc,
							out reset_freq  	flt_r_freq,
							out ir_index 		flt_ir_index,
							out flt_comp_avg_type 		flt_ca_t,
							out flt_sprd_comp_method 	flt_comp_m,
							out flt_avg_method 			flt_avg)
	{
		
		QL_FAIL_COND(!def.is_swap_fltois(), "instr_def not a float-ois swap", E_INIT);
		is_ccy = def.is_ccy_swap();
		error_info ee = error_info(true,true);

		ne = def.swap_not_exchange(ee);
		if(null(ne))
			ne 	= is_ccy ? NE_BOTH : NE_NONE;

		settle_code 	= date_code(def.settle_code());		
		ois_cpn_freq 	= coupon_freq_from_int(def.swap_coupon_freq_leg2(ee))  ;		
		ois_pmt_calendar= def.swap_calendar_leg2(ee);
		ois_pmt_bus_day = def.swap_pmt_bus_day_leg2(ee);
		ois_end_of_month= def.swap_eom_leg2(ee);		
		ois_ccy 		= def.swap_currency_leg2(ee);
		ois_dc 			= def.swap_dc_method_leg2(ee);
		ois_ir_index 	= def.swap_ir_index_leg2(ee);
		ois_ca_t		= def.swap_comp_avg_type_leg2(ee);
		ois_comp_m 		= def.swap_sprd_comp_method_leg2(ee);
		ois_avg 		= def.swap_avg_method_leg2(ee);
		ois_comp_factor_round_code 		= null<round_code>;
		ois_imp_rate_pcnt_round_code 	= null<round_code>;										
		ois_ir_rule 					= interest_rule.IR_ADJ; 																
		ois_pmt_delay 					= 0;															
		ois_arr_type 					= rfr_arrears_type.PLAIN;
		ois_arr_days 					= 0;
		ois_endog_df 					= true; //FIX_ME
		
		flt_pmt_calendar= def.swap_calendar_leg1(ee);
		flt_pmt_bus_day = def.swap_pmt_bus_day_leg1(ee);
		flt_end_of_month= def.swap_eom_leg1(ee);
		flt_ccy 		= def.swap_currency_leg1(ee);
		flt_dc 			= def.swap_dc_method_leg1(ee);
		flt_reset_style = RS_NORM;
		flt_cpn_freq 	= coupon_freq_from_int(def.swap_coupon_freq_leg1(ee))  ;
		flt_r_freq 		= reset_freq_from_int(def.swap_reset_freq_leg1(ee))  ;
		flt_ir_index 	= def.swap_ir_index_leg1(ee);
		flt_ca_t		= def.swap_comp_avg_type_leg1(ee);
		flt_comp_m 		= def.swap_sprd_comp_method_leg1(ee);
		flt_avg 		= def.swap_avg_method_leg1(ee);
	}

	/*-----------------------------------------------------------------------
	  func: swap_fixflt_plain
	  minimal constructor for single curr "screen"-swap,
	  ----------------------------------------------------------------------*/
	swap_fltois swap_fltois_plain_int(	swap_fltois 				sw_tmpl,
										string 						instrument_name,	
										date option(nullable) 		trade_date,
										date option(nullable) 		settle_date,	
										date option(nullable) 		start_date,
										string option(nullable) 	fwd_start_code,
										date 	option(nullable)	maturity,
										string 	option(nullable)	maturity_code,
										number option(nullable)		sprd_vs_ois ,//if null --> will be set to par 									
										number 	 					notional,																										
										logical 					pay_float,
										..disc_func option(nullable) df_disc_func,
										fwd_func option(nullable) 	flt_fwd_func,
										..disc_func option(nullable) flt_fwd_disc_func,
										fwd_func option(nullable) 	ois_fwd_func,
										..disc_func option(nullable) ois_fwddf_func,
										number option(nullable) 	flt_curr_fix,									
										logical 					reg_first_period,
										error_info option(nullable) error )
	{	
		try{
				
			logical is_ccy				= sw_tmpl.is_ccy_swap();
			QL_FAIL_COND(is_ccy, "constructor not applicable for currency swap", E_INIT);

			error_info ee = error_info(true,true);

			calendar ois_pmt_calendar,flt_pmt_calendar;
			bd_convention ois_pmt_bus_day,flt_pmt_bus_day;
			date_code settle_code;
			coupon_freq  ois_cpn_freq, flt_cpn_freq;
			notional_exchg_style ne;			
			logical ois_end_of_month,flt_end_of_month;
			string ois_ccy, flt_ccy ;
			day_count_method ois_dc, flt_dc;
			flt_reset_style flt_reset_style;
			reset_freq  flt_r_freq;
			ir_index ois_ir_index, flt_ir_index;
			flt_comp_avg_type flt_ca_t, ois_ca_t;
			flt_sprd_comp_method flt_comp_m,ois_comp_m;
			flt_avg_method 	flt_avg, ois_avg;

			round_code ois_comp_factor_round_code ;
			round_code  ois_imp_rate_pcnt_round_code;							
			interest_rule  ois_ir_rule ;															
			number  ois_pmt_delay 	;									
			rfr_arrears_type  ois_arr_type 	;
			number  ois_arr_days 	;
			logical ois_endog_df;
			
			if(!sw_tmpl.is_ext_swap(ee)) {
				unpack_instr_def(sw_tmpl.instr_def(ee),is_ccy,ne, settle_code,ois_cpn_freq,ois_ccy,ois_pmt_calendar, ois_pmt_bus_day,
								 ois_end_of_month, ois_dc,ois_ir_index,ois_ca_t, ois_comp_m, ois_avg,ois_comp_factor_round_code,ois_imp_rate_pcnt_round_code,
								ois_ir_rule,ois_pmt_delay,ois_arr_type,ois_arr_days,ois_endog_df,
								flt_reset_style, flt_cpn_freq, flt_ccy, flt_pmt_calendar, flt_pmt_bus_day, flt_end_of_month, flt_dc,
								flt_r_freq, flt_ir_index, flt_ca_t, flt_comp_m, flt_avg);				
			}
			else {
				
				ne 				= notional_exchg_style.NE_NONE;
				settle_code 	= date_code(sw_tmpl.settle_code());
				integer cfreq_ois, cfreq_flt;
				sw_tmpl.coupon_freq( cfreq_flt,cfreq_ois, ee);				
				ois_cpn_freq 	= coupon_freq_from_int(cfreq_ois)  ;
				flt_cpn_freq 	= coupon_freq_from_int(cfreq_flt)  ;				
				sw_tmpl.calendar(flt_pmt_calendar,ois_pmt_calendar,ee);
				sw_tmpl.pmt_bus_day(flt_pmt_bus_day,ois_pmt_bus_day,ee);
				sw_tmpl.eom(flt_end_of_month,ois_end_of_month,ee);
				sw_tmpl.dc_method(flt_dc,ois_dc,ee);
				sw_tmpl.currency(flt_ccy,ois_ccy,ee);						
				ois_ir_index 	= sw_tmpl.ir_index_ois_leg(ee);
				flt_ir_index 	= sw_tmpl.ir_index_flt_leg(ee);
				flt_reset_style = RS_NORM;
				flt_r_freq 		= reset_freq_from_int(sw_tmpl.reset_freq_flt_leg(ee))  ;
				
				flt_ca_t 		= null;
				ois_ca_t		= null;
				flt_comp_m		= null;
				ois_comp_m		= null;
				flt_avg			= null;
				ois_avg			= null;

				ois_comp_factor_round_code 		= null<round_code>;
				ois_imp_rate_pcnt_round_code 	= null<round_code>;										
				ois_ir_rule 					= interest_rule.IR_ADJ; 																
				ois_pmt_delay 					= 0;															
				ois_arr_type 					= rfr_arrears_type.PLAIN;
				ois_arr_days 					= 0;
				ois_endog_df 					= true; //FIX_ME
				
			}

			logical average_notional = null<logical>;
			number sub_factor 		= null<number>;
			number multi_factor 	= null<number>;
			number power_factor 	= null<number>;
			
			ql_swap_ois_def ois_def 	= ql_swap_ois_def("ois_def", ois_ccy, settle_code, ois_cpn_freq, ois_dc,flt_roll_method.FLR_STRAIGHT,ois_pmt_bus_day,
															ois_pmt_calendar,ois_end_of_month, ois_ir_index,null<quote_style>,null<round_code>,																		
															ois_comp_factor_round_code,ois_imp_rate_pcnt_round_code, ois_ir_rule, 																
															ois_pmt_delay, ois_pmt_delay, ois_pmt_delay, ois_pmt_delay, ois_arr_type,
														    ois_arr_days, ois_ca_t, ois_comp_m, ois_avg,ne,
														    average_notional,sub_factor,multi_factor,power_factor,ois_endog_df);
	
			ql_swap_float_def flt_def 	= ql_swap_float_def("float_def",flt_ccy,settle_code, flt_cpn_freq,flt_r_freq,flt_dc,flt_pmt_bus_day,
															flt_pmt_calendar,flt_end_of_month, flt_ca_t,flt_comp_m,flt_avg, flt_ir_index,flt_reset_style,ne);

			//---common----;
			CORE_INT_FIXFLT.swap_start_date_helper(settle_code,trade_date,flt_pmt_calendar,fwd_start_code,start_date,settle_date);
			QL_FAIL_COND(null(settle_date),"invalid start date", E_INIT);

			tenor_code mat_code = CORE_INT_FIXFLT.swap_maturity_helper(null<instr_def>,maturity, maturity_code);
			
			//---leg---
			
			date  ois_eff_date = null<date>;
			date  flt_eff_date = null<date>;
			date  ois_first_cpn_date = null<date>;
			date  ois_last_reg_date = null<date>;
			date  flt_first_cpn_date = null<date>;
			date  flt_last_reg_date = null<date>;			
			
			if(reg_first_period){
				
				schedule sched = schedule(	start_date,maturity,maturity_code,null<number>,flt_pmt_calendar,
											flt_pmt_bus_day,flt_pmt_bus_day,schedule_period_type.REGULAR,
											 schedule_period_type.NONE,flt_end_of_month,ois_eff_date,ois_first_cpn_date,ois_last_reg_date,0,0);

				ois_first_cpn_date 	= sched.first_date(false);
				ois_last_reg_date 	= sched.last_reg_date(false);
				ois_eff_date		= sched.start_date(true);
		
			}

			..disc_func flt_disc_func, ois_disc_func;			
			CORE_INT_FIXOIS.ois_df_fwd_helper(	is_ccy,df_disc_func,ois_fwd_func,ois_fwddf_func,ois_ir_index,ois_endog_df,trade_date,
												settle_date,maturity,maturity_code,flt_disc_func,ois_disc_func);
			

			number	flt_spread 		= 0;		
			integer	cpn_roll_day 	= 0;
			vector(date) fixing_dates = null<vector(date)>;	
			vector(number) fixing_rates= null<vector(number)>;
			number flt_fix_proxy 	= flt_curr_fix;
			number ois_fix_proxy 	= null<number>;
			logical  allow_fwd_fix	= false;
			logical ois_approx = true;

			number ois_notional	;
			number flt_notional	= notional;
			number ois_fx_rate,flt_fx_rate ;
			CORE_INT_FIXFLT.set_ccy(is_ccy,flt_notional, ois_notional, flt_fx_rate , ois_fx_rate);
					
			ql_ois_leg ois;
			if(null(sprd_vs_ois))
				ois 	= ql_ois_leg(ois_def,"ois_leg",trade_date,settle_date,start_date,maturity,mat_code,
									!pay_float,ois_notional,ois_eff_date,ois_first_cpn_date,ois_last_reg_date,0,	
									ois_disc_func,ois_fwd_func,cpn_roll_day,ois_fx_rate,
									 fixing_dates,fixing_rates,ois_fix_proxy,allow_fwd_fix,ois_approx);
			else
				ois 	= ql_ois_leg(ois_def,"ois_leg",trade_date,settle_date,start_date,maturity,mat_code,
									!pay_float,ois_notional,ois_eff_date,ois_first_cpn_date,ois_last_reg_date,sprd_vs_ois,	
									ois_disc_func,ois_fwd_func,cpn_roll_day,ois_fx_rate,
									 fixing_dates,fixing_rates,ois_fix_proxy,allow_fwd_fix,ois_approx);


			tenor_surface flt_tenor_surface;
			CORE_INT_FIXFLT.set_tenor_surface(trade_date, flt_disc_func, flt_fwd_func, flt_fwd_disc_func, flt_ir_index,flt_tenor_surface);
			
			ql_float_leg flt 		= ql_float_leg(flt_def,"float_leg",trade_date,settle_date,start_date,maturity,mat_code,
													pay_float,flt_notional,flt_eff_date,flt_first_cpn_date,flt_last_reg_date,flt_spread,flt_disc_func,	
													flt_tenor_surface,cpn_roll_day,flt_fx_rate, fixing_dates,fixing_rates,flt_fix_proxy,allow_fwd_fix);
			
			ql_fixed_income_swap sw = ql_fixed_income_swap(flt,ois,instrument_name);

			if(null(sprd_vs_ois)) {
				number ois_s	= sw.solver(0,SS_OIS_SPRD_LEG2,false);
				QL_FAIL_COND(null(ois_s),"error solving for ois leg spread", E_INIT);				
				
				ois		= ql_ois_leg(ois_def,"ois_leg",trade_date,settle_date,start_date,maturity,mat_code,
											!pay_float,ois_notional,ois_eff_date,ois_first_cpn_date,ois_last_reg_date,ois_s,	
											ois_disc_func,ois_fwd_func,cpn_roll_day,ois_fx_rate,
											fixing_dates,fixing_rates,ois_fix_proxy,allow_fwd_fix,ois_approx);
				sw 		= ql_fixed_income_swap(flt,ois,instrument_name);

			}

			return swap_fltois(sw, error);						
		}
		catch {
			CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois_plain");
			return null<swap_fltois>;
		}
	}
}

/*-----------------------------------------------------------------------
  func: swap_fltois_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois_plain(	swap_fltois 				sw,
								string 						instrument_name,	
								date option(nullable) 		trade_date,
								date option(nullable) 		settle_date,	
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,																												
								number option(nullable)		sprd_vs_ois ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_float,
								..disc_func option(nullable) disc_func,								
								fwd_func option(nullable) 	flt_fwd_func,
								fwd_func option(nullable) 	ois_fwd_func,
							    number option(nullable) 	flt_curr_fix, 							   
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{
	return CORE_INT_FLTOIS.swap_fltois_plain_int(sw,instrument_name,trade_date,settle_date,start_date,fwd_start_code,maturity,maturity_code,
								 sprd_vs_ois ,notional, pay_float,disc_func,flt_fwd_func, null<disc_func>,ois_fwd_func, null<disc_func>,
								 flt_curr_fix,false,error );
}

/*-----------------------------------------------------------------------
  func: swap_fltois_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois_plain(	swap_fltois 				sw,
								string 						instrument_name,	
								date option(nullable) 		trade_date,
								date option(nullable) 		settle_date,
								date option(nullable) 		start_date,			//start_date has precedence over fwd_start_code
								string option(nullable) 	fwd_start_code,
								date option(nullable)		maturity,			//maturity has precedence over mat_code
								string 	option(nullable)	maturity_code,																												
								number option(nullable)		sprd_vs_ois ,//if null --> will be set to par 									
								number 	 					notional,																										
								logical 					pay_float,
								..disc_func option(nullable) disc_func,								
								disc_func option(nullable) 	flt_disc_func,
								disc_func option(nullable) 	ois_disc_func,
							    number option(nullable) 	flt_curr_fix,
							   
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{
	return CORE_INT_FLTOIS.swap_fltois_plain_int(sw,instrument_name,trade_date,settle_date,start_date,fwd_start_code,maturity,maturity_code,
								 sprd_vs_ois ,notional, pay_float,disc_func,null<fwd_func>, flt_disc_func,null<fwd_func>, ois_disc_func,
								 flt_curr_fix,false,error );
}



/*-----------------------------------------------------------------------
  func: swap_fltois_plain
  minimal constructor for single curr "screen"-swap,
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois_plain(	instrument 					fltois_sw,															
								number 	 					notional,																										
								logical 					pay_float,
								..disc_func option(nullable) disc_func,								
								fwd_func option(nullable) 	flt_fwd_func,
								fwd_func option(nullable) 	ois_fwd_func,
							    number option(nullable) 	flt_curr_fix,							    
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
{
	error_info ee 			= new error_info(true,true);
	QL_FAIL_COND(fltois_sw.instr_type(ee) != instr_type.SWAP_FLTOIS, "invalid instrument type");
	swap_fltois sw 			= swap_fltois(fltois_sw,ee);
	string instrument_name 	= sw.name(ee);	
	date trade_date 		= sw.trade_date(ee);
	date settle_date 		= sw.settle_date(ee);
	date start_date 		= sw.issue_date(ee);
	string fwd_start_code	= null<string>;
	date maturity 			= sw.maturity(ee);			
	string maturity_code 	= null<string>;																												
	number sprd 			= sw.quote(ee)/10000.0;
	return CORE_INT_FLTOIS.swap_fltois_plain_int(sw,instrument_name,trade_date,settle_date,start_date,fwd_start_code,maturity,maturity_code,
												sprd ,notional, pay_float,disc_func,flt_fwd_func, null<disc_func>,ois_fwd_func, null<disc_func>,
												flt_curr_fix, false,error );
}

/*-----------------------------------------------------------------------
  func: swap_fixois_comb
  ----------------------------------------------------------------------*/
swap_fixois swap_fixois_comb(	swap_fixflt 				sw_fixflt,
								swap_fltois 				sw_fltois,
								string 						instrument_name,																																					
								number option(nullable)		sprd_vs_ois ,																								   
								error_info option(nullable) error = null<error_info>)
option (category: 'Instrument/Interest Rate Swap/Fixed vs ONIndex')
{
	error_info ee 		= new error_info(true,true);
	//ql_float_leg flt 	= sw_fixflt.float_leg(false,ee);
	ql_fix_leg fix 		= sw_fixflt.fix_leg(true,ee);

	QL_REQUIRE(sw_fixflt.trade_date(ee) 			== sw_fltois.trade_date(ee),"float leg mismatch: trade_date");
	QL_REQUIRE(sw_fixflt.settle_date(ee) 			== sw_fltois.settle_date(ee),"float leg mismatch: settle_date");
	QL_REQUIRE(sw_fixflt.maturity(ee) 				== sw_fltois.maturity(ee),"float leg mismatch: maturity");
	QL_REQUIRE(sw_fixflt.coupon_freq_flt_leg(ee) 	== sw_fltois.coupon_freq_flt_leg(ee),"float leg mismatch: coupon frequency");
	QL_REQUIRE(sw_fixflt.reset_freq_flt_leg(ee) 	== sw_fltois.reset_freq_flt_leg(ee),"float leg mismatch: reset frequency");
	QL_REQUIRE(sw_fixflt.spread_flt_leg(ee) 		== sw_fltois.spread_flt_leg(ee),"float leg mismatch: float spread");
	QL_REQUIRE(sw_fixflt.currency_flt_leg(ee) 		== sw_fltois.currency_flt_leg(ee),"float leg mismatch: currency");
	//compare float leg
	//ql_float_leg flt_o 	= sw_fltois.float_leg(false,ee);
	ql_ois_leg ois 		= sw_fltois.ois_leg(true,ee);
	
	ql_fixed_income_swap qlsw = ql_fixed_income_swap(fix,ois,instrument_name);
	swap_fixois sw 		= swap_fixois(qlsw, ee);
	sw.set_spread_ois_leg(sprd_vs_ois,ee);

	return sw;
}