option(null: hard);	
/*	
	instrument wrapper functions
	Developer: Algorithmica Research, Magnus Nyström			

	--------------------------------------------------------------------
	fltois
	Note:
	for db-swaps: leg1 is the float leg i.e. the swap is fltleg + quote = oisleg, floatleg is specified in the general tab
	--------------------------------------------------------------------
*/

//----------------------------------------------------------------------
//  class swap_fltois
//----------------------------------------------------------------------
class swap_fltois: public swap_gen
option(category: "Instrument/Interest Rate Swap/Float vs ONIndex")
//option(allow_undeclared_mfuncs)
{
public:
    //override string	instr_type_s();
	override instrument 	inst();
	override swap_fltois 	clone() ;	
	override void         __dbg_print(__dbg_label);
    override void         __dbg_browse(__dbg_split);
	
	/*----------ADD FUNCS [PUBLIC]---------------*/
	void 		add_quote(number,..quote_style );
	override void add_quote(number);
	void 		add_notional(number );
	void 		add_notional_flt_leg(number );
	void 		add_notional_ois_leg(number );
	void 		add_fx_rate_flt_leg(number );
	void 		add_fx_rate_ois_leg(number );
	
	void 		add_disc_func(disc_func, logical disable_endog_df = false ); 
	void 		add_disc_func_flt_leg(disc_func );	
	void 		add_disc_func_ois_leg(disc_func, logical disable_endog_df = false  );	
	void		add_fwd_func_ois_leg(fwd_func, logical disable_endog_df = false );
	void 		add_surface_flt_leg(tenor_surface option(nullable));
	void 		add_surface_ois_leg(tenor_surface option(nullable));//legacy and hidden
	
	
	
	void		add_fixing_flt_leg(vector(date) ,vector(number) ,logical option(nullable) fixing_as_fwd = false);
	void		add_fixing_ois_leg(vector(date) ,vector(number) ,logical option(nullable) fixing_as_fwd = false);
	void		add_fixing_flt_leg(number ,logical option(nullable) fixing_as_fwd = false);
	void		add_fixing_ois_leg(number ,logical option(nullable) fixing_as_fwd = false);
	void		add_current_fixing_flt_leg(number option(nullable));

	/*
	 add_data()
	 add_ccy_data()
	 */
	
	/*----------MISC FUNCS [PUBLIC]---------------*/
	date 				start_date(error_info option(nullable) error = null<error_info> );				/*name change of issuedate*/

	//override void 		is_amortizing(out logical,out logical ,error_info option(nullable) error = null<error_info>);
	//override logical 	is_amortizing(error_info option(nullable) error = null<error_info>);
	//logical				is_par_flat(error_info option(nullable) error = null<error_info>);
	logical 			pay_float(error_info option(nullable) error = null<error_info> );
	override void 		notional(out number , out number , error_info option(nullable) error = null<error_info>);
	number 				notional(error_info option(nullable) error = null<error_info>);
	//void 				is_regular_first(out logical , out logical , error_info option(nullable) error = null<error_info>);				
	//void 				is_regular_last(out logical , out logical , error_info option(nullable) error = null<error_info>);		
	//override void 		fx_rate(out number , out number , error_info option(nullable) error = null<error_info>);	
	override void 		currency(out string , out string ,error_info option(nullable) error = null<error_info> );
	//override void 		coupon(out number ,out number , error_info option(nullable) error = null<error_info>);
	override void 		coupon_freq(out integer,out integer, error_info option(nullable) error = null<error_info> );
	override void 		dc_method(	out day_count_method , out day_count_method ,error_info option(nullable) error = null<error_info>);	
	override void  		pmt_bus_day(out bd_convention , out bd_convention ,error_info option(nullable) error = null<error_info> );
	//override void 		roll_day(out integer,out integer,  error_info option(nullable) error = null<error_info>);
	override void 		eom(out logical ,out logical ,error_info option(nullable) error = null<error_info> );
	override void 		calendar(out ..calendar ,out ..calendar ,error_info option(nullable) error = null<error_info> );
	//override void  		first_cpn_date(	out date , out date , error_info option(nullable) error = null<error_info>);
	//override void  		last_reg_cpn_date(	out date , out date , error_info option(nullable) error = null<error_info>);
	//override void  		effective_date(	out date , out date , error_info option(nullable) error = null<error_info>);
	//override void  		next_cpn_date(	out date , out date , error_info option(nullable) error = null<error_info>);
	//override void  		previous_cpn_date(	out date , out date , error_info option(nullable) error = null<error_info>);
	//override void		calendar_name(out string ,out string  , error_info option(nullable) error = null<error_info> ) ;
	//override string 	settle_code(error_info option(nullable) error = null<error_info>);
	
	override void 		reset_freq(	out integer , out integer  , error_info option(nullable) error = null<error_info>) ;
	logical				flat_leg_is_float_leg(error_info option(nullable) error = null<error_info> );
	logical				flat_leg_is_ois_leg(error_info option(nullable) error = null<error_info> );

	override void 		quote(out number  ,out number  ,error_info option(nullable) error = null<error_info> );	
	override number 	quote(error_info option(nullable) error = null<error_info> );
	
	/*----------CASHFLOW FUNCS [PUBLIC]---------------*/
	override void		cash_flow_dates(logical,out vector(date) ,out vector(date) ,error_info option(nullable) error = null<error_info> ) ;
	void				prin_cash_flow_dates(logical,out vector(date) ,out vector(date) ,error_info option(nullable) error = null<error_info> ) ;
	
	/*----------PV FUNCS [PUBLIC]---------------*/
	override number  	present_value(	disc_func option(nullable),disc_func option(nullable),out number ,
								out number ,number option(nullable)=null<number>,logical option(nullable) trade_date_pv = false,		
								error_info option(nullable) error = null<error_info>);
	
	override number  	present_value(	disc_func option(nullable),disc_func option(nullable),out number ,out number ,
								date option(nullable) ,date option(nullable) ,
								logical,logical  ,error_info option(nullable) error = null<error_info>);
	
	override number  	present_value(	out number ,out number pv_ois_leg,
								logical option(nullable) ,error_info option(nullable) error = null<error_info> );

	override number		solver(	number,swap_solver_code ,logical option(nullable) trade_date_pv = false,
						error_info option(nullable) error = null<error_info>);

	/*----------FLTLEG FUNCS [PUBLIC]---------------*/
	disc_func 			disc_func_flt_leg(error_info option(nullable) error = null<error_info> ) ;
	void				accr_dates_flt_leg(	logical,out vector(date),out vector(date),out vector(number) ,
												error_info option(nullable) error= null<error_info> );
	
	ir_index 			ir_index_flt_leg(error_info option(nullable) error = null<error_info>);
	 
	tenor_code 			index_tenor_code_flt_leg(error_info option(nullable) error = null<error_info> ) ;
	string 				currency_flt_leg(error_info option(nullable) error = null<error_info>);
	string 				index_tenor_flt_leg(error_info option(nullable) error = null<error_info>) ;
	number				spread_flt_leg(error_info option(nullable) error = null<error_info> );
	integer				reset_freq_flt_leg(error_info option(nullable) error = null<error_info>);
	integer				coupon_freq_flt_leg(error_info option(nullable) error = null<error_info>);
	number				current_coupon_flt_leg(error_info option(nullable) error = null<error_info> );
	number				current_fixing_flt_leg(out date,out logical,error_info option(nullable) error = null<error_info>);
	day_count_method  	dc_method_flt_leg( error_info option(nullable) error = null<error_info>);

	/*
	number					pv01_spread(number option(nullable)= null<number>,number option(nullable)= null<number>,
										disc_func option(nullable) df = null<disc_func>,
										error_info option(nullable) error = null<error_info>);

	void					index_dates_flt_leg(logical,out vector(date) ,out vector(date) ,
												out vector(logical) ,error_info option(nullable) error = null<error_info> ) ;

		
	void					fixing_data_flt_leg(logical ,logical option(nullable),out vector(date)  ,									
												out vector(number)  ,out vector(logical),
												error_info option(nullable) error = null<error_info> ) ;
	number					imp_spread_flt_leg(number,logical option(nullable) trade_date_pv = false,
											   error_info option(nullable) error = null<error_info>);
	
	disc_func 				disc_func_flt_leg(error_info option(nullable) error = null<error_info> ) ;
	tenor_surface 			tenor_surface_flt_leg(error_info option(nullable) error = null<error_info>);
	
	
	tenor_surface 			create_tenor_surface(disc_func ,fwd_func,error_info option(nullable) error = null<error_info> );
	tenor_surface 			create_tenor_surface(disc_func ,disc_func,error_info option(nullable) error = null<error_info> );
	*/
	
	
	/*----------OISLEG FUNCS [PUBLIC]---------------*/
	
	disc_func 			disc_func_ois_leg(error_info option(nullable) error = null<error_info> ) ;
	fwd_func 			fwd_func_ois_leg(error_info option(nullable) error = null<error_info>);
	void				accr_dates_ois_leg(	logical,out vector(date),out vector(date),out vector(number) ,
												error_info option(nullable) error= null<error_info>); 
	ir_index 			ir_index_ois_leg(error_info option(nullable) error = null<error_info> );
	number				spread_ois_leg(error_info option(nullable) error = null<error_info> );
	string 				currency_ois_leg(error_info option(nullable) error = null<error_info>);
	logical  			is_endog_df(error_info option(nullable) error = null<error_info>);
	integer				reset_freq_ois_leg(error_info option(nullable) error = null<error_info>);
	
	/*-----external swap specific-----*/
	override ql_fixed_income_swap 	swap_instrument(error_info option(nullable) error = null<error_info>) ;
	override instrument_type 		swap_type(error_info option(nullable) error = null<error_info>);
	vector(ql_swap_leg) 	legs(logical,error_info option(nullable) error = null<error_info>);
	ql_float_leg 			float_leg(logical,error_info option(nullable) error = null<error_info>);
	ql_ois_leg 				ois_leg(logical,error_info option(nullable) error = null<error_info>);


	/*---set functions-required--*/	
	override swap_fltois	set_quote(number option(nullable),error_info option(nullable) error = null<error_info>);
	override swap_fltois	set_quote(string option(nullable),error_info option(nullable) error = null<error_info>);
	
	
	swap_fltois				set_spread_ois_leg(number option(nullable) ,error_info option(nullable) error = null<error_info>);
	swap_fltois				set_spread_flt_leg(number option(nullable) ,error_info option(nullable) error = null<error_info>);

	//local set funcs
	//swap_fixois 			set_date(date,date option(nullable) settle_date=null<date>, error_info option(nullable) error = null<error_info>);
	//override swap_fixois	set_date(date,date option(nullable),logical,logical ,error_info option(nullable) error = null<error_info> );
	//void					update_to_par(error_info option(nullable) error = null<error_info> );
	swap_fltois				set_pay_leg(logical ,error_info option(nullable) error = null<error_info>);
	//--------------------------------------------------------------------------
	//----------HIDDEN FUNCS [PUBLIC]--(mostly for backward compat)-------------
	//--------------------------------------------------------------------------
	swap_fltois(__instrument);
	//general-one per leg
	override string 		currency(error_info option(nullable) error = null<error_info> );				/*hidden, visible in root*/
	override logical 		eom(error_info option(nullable) error = null<error_info>);						/*hidden, not visible in root*/
	
	swap_fltois(swap_fltois );

protected:
	
	override swap_fltois	create(__instrument option(nullable),out instr_error option(nullable),error_type type = E_INVALID_ARG);
	swap_fltois 			err_type(swap_fltois option(nullable), error_info option(nullable) error = null<error_info>);
	
	void					SWAP_ENTRY_FUNC();

	void 					add_disc_func_int(disc_func,logical  );
	void 					add_disc_func_int_ois_leg(disc_func,logical );
	void					add_fwd_func_int_ois_leg(fwd_func,logical );
	
	logical ext_swap_;
	logical ccy_swap_;
	logical full_swap_;
};

//------------------------------------------------
// __dbg_print
//------------------------------------------------
void swap_fltois.__dbg_print(__dbg_label l)
{

	error_info ee 	= new error_info(true,false);
    l.set_text(strcat([
						"name: ",
						name(ee),
						",  ext_swap: ",
						null(ext_swap_) ? "" : string(ext_swap_),
                        ", ccy_swap: ",
						null(ccy_swap_) ? "" : string(ccy_swap_),
						", valid: ",
						null(this.is_valid(ee)) ? "" : string(this.is_valid(ee))
					]));
}
//------------------------------------------------
// __dbg_browse
//------------------------------------------------
void swap_fltois.__dbg_browse(__dbg_split s)
{
    s.resize(4);
	error_info ee 	= new error_info(true,false);
    s.set_text(0, "name");
    s.set_value(0, this.name(ee));

    s.set_text(1, "ext_swap");
    s.set_value(1, ext_swap_);

    s.set_text(2, "ccy_swap");
    s.set_value(2, ccy_swap_);

	s.set_text(3, "valid");
    s.set_value(3, is_valid(ee));
}

//------------------------------------------------
// constructor [HIDDEN, cannot be protected]
//------------------------------------------------
swap_fltois.swap_fltois(__instrument i) option(hidden): swap_gen(i)
{
	//try {
		error_info ee 	= new error_info(true,true);
		if(!this.is_valid(ee)) {
			ext_swap_ = ccy_swap_ = full_swap_ = false;
			return;
		}
		ext_swap_ 		= this.is_ext_swap(ee);
		ccy_swap_ 		= this.is_ccy_swap(ee);
		//FIX_ME 
		//QL_REQUIRE(!ccy_swap_,"flt-ois cross currency swap is currently work in progress");
		full_swap_ 		= this.verify_full_swap(ee);
		return;									
	//}
	//catch {		
	//	CORE_INT.write_warning_message(err.message());
	//}
}
//------------------------------------------------
// instr_type_s
//------------------------------------------------
//string swap_fltois.instr_type_s() 	{ return string(..instr_type.SWAP_FLTOIS);}
//------------------------------------------------
// inst
//------------------------------------------------
instrument 	swap_fltois.inst()		{ return this;}
//------------------------------------------------
// copy constructor
//------------------------------------------------
swap_fltois.swap_fltois(swap_fltois c) : swap_gen(c),
						  ext_swap_(c.ext_swap_), ccy_swap_(c.ccy_swap_), full_swap_(c.full_swap_) {}
//------------------------------------------------
// copy constructor <FUNCTION>
//------------------------------------------------
swap_fltois swap_fltois(swap_fltois b)			{ return new swap_fltois(b);}
//------------------------------------------------
// clone
//------------------------------------------------
swap_fltois swap_fltois.clone() 	{ return new swap_fltois(this);}
//-----------------------------------------------
// dynamic cast <FUNCTION>   EXT SWAP: ok
//-----------------------------------------------
swap_fltois swap_fltois(instrument i, error_info option(nullable) error = null<error_info>) 	
option (category: 'Instrument/Interest Rate Swap/Float vs ONIndex')
option(com_name: 'swap_fltois_dyncast')
{ 
	try {
		swap_fltois d = dynamic_cast<swap_fltois>(i); 
		if(null(d))
			CORE_INT.add_error_info(error,ERR_T_INIT,"invalid cast (instrument is not a swap_fltois)","swap_fltois" );

		return d;
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois");
		return null<swap_fltois>;
	}
}
//------------------------------------------------
// create  [PROTECTED]
// create swap_fltois from internal instrument
//------------------------------------------------
swap_fltois swap_fltois.create(	__instrument option(nullable) c,
								out instr_error option(nullable) error,
								error_type type )
{
	QL_FAIL_COND(null(c), "invalid/unknown (null) instrument",type );

	logical valid = c.is_valid(error);
	if( !valid && INSTR_CREATE_NULL_ERR)
		return null<swap_fltois>; 
	
	..instr_type it = c.instr_type(error);
	QL_FAIL_COND(null(it), "invalid instrument type (unknown)",type );
	QL_FAIL_COND(it != ..instr_type.SWAP_FLTOIS, "invalid instrument type",type );//should not happen
	
	return new swap_fltois(c);
}

/*-----------------------------------------------------------------------
  function: create_swap_fltois
  ----------------------------------------------------------------------*/
swap_fltois create_swap_fltois(	__instrument option(nullable) c,
									out instr_error option(nullable) error,
									error_type type = E_INVALID_ARG)
option(com_name: 'INTERNAL_create_swap_fltois')
{
	QL_FAIL_COND(null(c), "invalid/unknown (null) instrument",type );

	logical valid = c.is_valid(error);
	if( !valid && INSTR_CREATE_NULL_ERR)
		return null<swap_fltois>; 
	
	..instr_type it = c.instr_type(error);
	QL_FAIL_COND(null(it), "invalid instrument type (unknown)",type );
	QL_FAIL_COND(it != ..instr_type.SWAP_FLTOIS, "invalid instrument type",type );//should not happen
	
	return new swap_fltois(c);
}
swap_fltois create_swap_fltois(	__instrument option(nullable) 		c,
								error_info option(nullable) 		error,
								error_type 							type = E_INIT)
option(com_name: 'INTERNAL_create_swap_fltois_ei')
{
	instr_error ee 	= instr_error();
	swap_fltois b 	= create_swap_fltois(c,ee, E_INIT);
	if(ee.is_error())
		CORE_INT.add_error_info(error,ee.type(),ee.message(), "create_swap_fltois");
	return b;
}

/*-----------------------------------------------------------------------
  ENTRY_FUNC prot
  ----------------------------------------------------------------------*/
void	swap_fltois.SWAP_ENTRY_FUNC()
{
	if(!null(full_swap_) && full_swap_)
		return;
	error_info ee = new error_info(true,true);
	full_swap_ = check_full_swap(full_swap_, ee) ;	
}
/*-----------------------------------------------------------------------
  err_type	EXT SWAP: ok
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois.err_type(swap_fltois option(nullable) nul,
								 error_info option(nullable) error )
{	
	CORE_INT.add_error_info(error,ERR_T_CALC,"mwlib(ql) error: inapplicable function call for current instrument type");
	return null<swap_fltois>;
}
/*-----------------------------------------------------------------------
  swap_fltois: add_funcs
  ----------------------------------------------------------------------*/
void swap_fltois.add_notional(number notional)				{ SWAP_ENTRY_FUNC();swap_gen.swap_add_notional(notional);}
void swap_fltois.add_notional_flt_leg(number notional) 		{ SWAP_ENTRY_FUNC();swap_gen.swap_add_notional_leg1(notional);}
void swap_fltois.add_notional_ois_leg(number notional)		{ SWAP_ENTRY_FUNC();swap_gen.swap_add_notional_leg2(notional);	}
void swap_fltois.add_fx_rate_flt_leg(number fx_rate)		{ SWAP_ENTRY_FUNC();swap_gen.swap_add_fx_rate_leg1(fx_rate);}
void swap_fltois.add_fx_rate_ois_leg(number fx_rate)		{ SWAP_ENTRY_FUNC();swap_gen.swap_add_fx_rate_leg2(fx_rate);}
void swap_fltois.add_disc_func_flt_leg(disc_func disc_func) { SWAP_ENTRY_FUNC();swap_gen.swap_add_disc_func_leg1(disc_func); }
void swap_fltois.add_current_fixing_flt_leg(number option(nullable) fixing) 	{ SWAP_ENTRY_FUNC();swap_gen.swap_add_curr_fixing_leg1(fixing);	}
/*-----------------------------------------------------------------------
  swap_fltois: add_quote		EXT SWAP: ok
  ----------------------------------------------------------------------*/
void swap_fltois.add_quote(	number  quote ) 
{
	SWAP_ENTRY_FUNC();
	instrument.add_quote(quote, false);
	return ;						
}
/*-----------------------------------------------------------------------
  swap_fltois: add_quote		EXT SWAP: ok
  ----------------------------------------------------------------------*/
void swap_fltois.add_quote(	number  quote,
							..quote_style quote_style) 
{
	SWAP_ENTRY_FUNC();
	error_info ee = new error_info(true,true);
	..quote_style qs = instrument.quote_style_e(ee);
	
	if(quote_style == qs)
		this.add_quote(quote);
	else{
		if(quote_style == ..quote_style.SPREAD)
			this.add_quote(quote*10000);//instr quotestyle must be SPREAD_BP
		else if(quote_style == ..quote_style.SPREAD_BP)
			this.add_quote(quote/10000);//instr quotestyle is SPREAD
		else
			QL_FAIL("invalid quote style",this,true);
	}					
}
/*-----------------------------------------------------------------------
  add_disc_func		EXT SWAP: ok
  ----------------------------------------------------------------------*/
void swap_fltois.add_disc_func(disc_func disc_func, logical disable_endog_df)
{
	//swap_gen.swap_add_disc_func(disc_func);
	QL_REQUIRE(!ccy_swap_,"inapplicable function call for cross currency swaps",this,true);
	if(disable_endog_df)
		add_disc_func_int_ois_leg(disc_func,false);
	else
		add_disc_func_ois_leg(disc_func);
	add_disc_func_flt_leg(disc_func);
}

void swap_fltois.add_disc_func_int(disc_func disc_func, logical is_endog)
{
	QL_REQUIRE(!ccy_swap_,"inapplicable function call for cross currency swaps",this,true);
	add_disc_func_int_ois_leg(disc_func, is_endog);
	add_disc_func_flt_leg(disc_func);
}

void swap_fltois.add_disc_func_int_ois_leg(disc_func  disc_func, logical is_endog)
{
	SWAP_ENTRY_FUNC();
	error_info ee = new error_info(true,true);
	if(is_endog){
		..ir_index idx = ir_index(ee);
		fwd_func f = fwd_func(disc_func,365,trade_date(),idx.day_count_method(),idx.fixing_calendar(),BD_FOLLOWING,false);
		swap_gen.swap_add_fwd_func_leg2(f);
	}
	
	swap_gen.swap_add_disc_func_leg2(disc_func);
	//if(!ccy_swap_)
		//swap_gen.swap_add_disc_func_leg1(disc_func);
}
void swap_fltois.add_fwd_func_int_ois_leg(fwd_func fwd_func, logical is_endog)
{
	SWAP_ENTRY_FUNC();
	error_info ee = new error_info(true,true);
	if(is_endog){
		date m = maturity(ee);
		..ir_index idx = ir_index(ee);
		disc_func df = CORE_INT.on_fwd_to_df(trade_date(),m,fwd_func,idx.day_count_method(),idx.fixing_calendar(),true);		
		swap_gen.swap_add_disc_func_leg2(df);
		//if(!ccy_swap_)
			//swap_gen.swap_add_disc_func_leg1(df);
	}
	swap_gen.swap_add_fwd_func_leg2(fwd_func);
}

void swap_fltois.add_fwd_func_ois_leg(fwd_func fwd_func, logical disable_endog_df)
{
	error_info ee = new error_info(true,true);
	if(disable_endog_df)
		add_fwd_func_int_ois_leg(fwd_func , false);
	else
		add_fwd_func_int_ois_leg(fwd_func , this.is_endog_df(ee));
}

void swap_fltois.add_disc_func_ois_leg(disc_func  disc_func,
									   logical disable_endog_df)
{
	//swap_gen.swap_add_disc_func_leg2(disc_func);
	error_info ee = new error_info(true,true);
	if(disable_endog_df)
		add_disc_func_int_ois_leg(disc_func, false);
	else
		add_disc_func_int_ois_leg(disc_func, this.is_endog_df(ee));
}

void swap_fltois.add_surface_flt_leg(tenor_surface option(nullable) tenor_surface)	{ swap_gen.swap_add_surface_leg1(tenor_surface);}
void swap_fltois.add_fixing_flt_leg(	vector(date) fixing_dates,
									vector(number) fixings,
									logical option(nullable) fixing_as_fwd )
{			
	swap_gen.swap_add_fixing_leg1(fixing_dates,fixings,fixing_as_fwd);						
}
void swap_fltois.add_surface_ois_leg(tenor_surface option(nullable) tenor_surface)	option(hidden)
{
	SWAP_ENTRY_FUNC();
	error_info ee = new error_info(true,true);
	QL_REQUIRE(this.is_endog_df(ee),"function not supported for fixois without self-discounted ois-leg",this,true);
	swap_gen.swap_add_surface_leg2(tenor_surface);
}

void swap_fltois.add_fixing_ois_leg(vector(date) fixing_dates,
									vector(number) fixings,
									logical option(nullable) fixing_as_fwd )
{			
	swap_gen.swap_add_fixing_leg2(fixing_dates,fixings,fixing_as_fwd);						
}
void swap_fltois.add_fixing_flt_leg(	number fixing_proxy,
									logical option(nullable) fixing_as_fwd ) 
{			
	swap_gen.swap_add_fixing_leg1(fixing_proxy, fixing_as_fwd);						
}
void swap_fltois.add_fixing_ois_leg(	number fixing_proxy,
									logical option(nullable) fixing_as_fwd ) 
{			
	swap_gen.swap_add_fixing_leg2(fixing_proxy, fixing_as_fwd);						
}
/*-----------------------------------------------------------------------
  swap_fltois: disc_func_flt_leg / disc_func_ois_leg
  ----------------------------------------------------------------------*/
disc_func swap_fltois.disc_func_flt_leg(error_info option(nullable) error) 	{ return swap_gen.swap_disc_func_leg1(error);}
disc_func swap_fltois.disc_func_ois_leg(error_info option(nullable) error) 	{ return swap_gen.swap_disc_func_leg2(error);}

/*-----------------------------------------------------------------------
  flat_leg_is_float_leg
  ----------------------------------------------------------------------*/
logical	swap_fltois.flat_leg_is_float_leg(error_info option(nullable) error  )
{	
	return !swap_gen.leg1_is_quote(error);
}
/*-----------------------------------------------------------------------
  flat_leg_is_ois_leg
  ----------------------------------------------------------------------*/
logical	swap_fltois.flat_leg_is_ois_leg(error_info option(nullable) error )
{	
	return swap_gen.leg1_is_quote(error);
}	
/*-----------------------------------------------------------------------
  swap_type
  ----------------------------------------------------------------------*/
instrument_type swap_fltois.swap_type(error_info option(nullable) error) 
{	
	return swap_gen.swap_type(error);
}

/*-----------------------------------------------------------------------
  solver
  ----------------------------------------------------------------------*/
number swap_fltois.solver(	number 				pv,
							swap_solver_code 	solver_code,
							logical option(nullable) trade_date_pv,
							error_info option(nullable) error)
{	
	return swap_gen.solver(pv,solver_code,trade_date_pv,error);
}

/*-----------------------------------------------------------------------
  swap_instrument
  ----------------------------------------------------------------------*/
ql_fixed_income_swap swap_fltois.swap_instrument(error_info option(nullable) error)
{
	return swap_gen.swap_instrument(error);

}
/*-----------------------------------------------------------------------
  index_tenor_code_leg1
  ----------------------------------------------------------------------*/
tenor_code swap_fltois.index_tenor_code_flt_leg(error_info option(nullable) error) 
{	
	return swap_gen.swap_index_tenor_code_leg1(error);
}
/*-----------------------------------------------------------------------
  current_fixing_flt_leg	EXT SWAP: ok
  ----------------------------------------------------------------------*/
number	swap_fltois.current_fixing_flt_leg(out date reset_date,
										   out logical is_fixed,
										   error_info option(nullable) error)
{
	try{
		error_info ee = error_info(true,true);
		ql_float_leg flt = this.float_leg(false,ee);

		return flt.current_fixing_rate(reset_date,is_fixed);
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.current_fixing_flt_leg");		
		return null<number>;
	}
}
/*-----------------------------------------------------------------------
  current_coupon_flt_leg	DLL_SWAP_HANDLER: ok
  ----------------------------------------------------------------------*/
number  swap_fltois.current_coupon_flt_leg(error_info option(nullable) error ) 
{
	number  cpn_leg1 ,cpn_leg2;
	swap_gen.coupon(cpn_leg1 ,cpn_leg2,  error );
	return cpn_leg2;
}
/*-----------------------------------------------------------------------
  reset_freq_flt_leg	
  ----------------------------------------------------------------------*/
integer  swap_fltois.reset_freq_flt_leg(error_info option(nullable) error ) 
{
	integer c1,c2;
	swap_gen.reset_freq(c1,c2  ,error ) ;
	return c1;
}
/*-----------------------------------------------------------------------
  coupon_freq_flt_leg	
  ----------------------------------------------------------------------*/
integer  swap_fltois.coupon_freq_flt_leg(error_info option(nullable) error ) 
{
	integer c1,c2;
	swap_gen.coupon_freq(c1,c2  ,error ) ;
	return c2;
}
/*-----------------------------------------------------------------------
  coupon_freq   	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.coupon_freq(	out integer  cpn_freq_flt_leg, 
								out integer  cpn_freq_ois_leg, 
								error_info option(nullable) error ) 
{	
	try{	
		instr_error_type t;
   		string 			s;	
		i().__swap_cpn_freq(cpn_freq_flt_leg,cpn_freq_ois_leg,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.coupon_freq");
		if(e)
			cpn_freq_flt_leg = cpn_freq_ois_leg = err_freq();
		return ;						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.coupon_freq");
		cpn_freq_flt_leg = cpn_freq_ois_leg = err_freq();
		return ;
	}
}
/*-----------------------------------------------------------------------
  dc_method   	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.dc_method(	out day_count_method  dc_method_flt_leg, 
								out day_count_method  dc_method_ois_leg, 
								error_info option(nullable) error ) 
{	
	try{	
		instr_error_type t;
   		string 			s;	
		i().__swap_dc_method(dc_method_flt_leg,dc_method_ois_leg,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.dc_method");
		if(e)
			dc_method_flt_leg = dc_method_ois_leg = null<day_count_method>;
		return ;					
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.dc_method");
		dc_method_flt_leg = dc_method_ois_leg = null<day_count_method>;
		return ;
	}
}
/*-----------------------------------------------------------------------
  pmt_bus_day 	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.pmt_bus_day(	out bd_convention  pmt_bus_day_flt_leg, 
								out bd_convention  pmt_bus_day_ois_leg, 
								error_info option(nullable) error) 
{	
	try{	
		instr_error_type t;
   		string 			s;	
		i().__swap_bd(pmt_bus_day_flt_leg,pmt_bus_day_ois_leg,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.pmt_bus_day");
		if(e)
			pmt_bus_day_flt_leg = pmt_bus_day_ois_leg = null<bd_convention>;
		return ;					
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.pmt_bus_day");		
		pmt_bus_day_flt_leg = pmt_bus_day_ois_leg = null<bd_convention>;
		return ;
	}
}
/*-----------------------------------------------------------------------
  eom	EXT SWAP: ok
  ----------------------------------------------------------------------*/
logical swap_fltois.eom(error_info option(nullable) error) option(hidden)
{	
	try{	
		instr_error_type t;
   		string 			s;	
		logical c1,c2;
		i().__swap_eom(c1,c2,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.eom");	
		return e ? null<logical>: c1;							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.eom");		
		return null<logical>;
	}
}
/*-----------------------------------------------------------------------
  eom   	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.eom(	out logical   eom_flt_leg, 
						out logical  eom_ois_leg, 
						error_info option(nullable) error) 
{	
	try{	
		instr_error_type t;
   		string 			s;	
		i().__swap_eom(eom_flt_leg,eom_ois_leg,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.eom");
		if(e)
			eom_flt_leg = eom_ois_leg = null<logical>;
		return ;						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.eom");		
		eom_flt_leg = eom_ois_leg = null<logical>;
		return ;
	}
}

/*-----------------------------------------------------------------------
  cal   	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.calendar(	out ..calendar   cal_flt_leg, 
							out ..calendar  cal_ois_leg, 
							error_info option(nullable) error) 
{	
	try{	
		instr_error_type t;
   		string 			s;	
		i().__swap_calendar(cal_flt_leg,cal_ois_leg,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.calendar");
		if(e)
			cal_flt_leg = cal_ois_leg = null<..calendar>;
		return ;						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.calendar");		
		cal_flt_leg = cal_ois_leg = null<..calendar>;
		return ;
	}
}
/*-----------------------------------------------------------------------
  quote 	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.quote(	out number  spread_flt_leg, 
							out number  spread_ois_leg, 
							error_info option(nullable) error ) 
{	
	swap_gen.quote(spread_flt_leg,  spread_ois_leg, error);
}
/*-----------------------------------------------------------------------
  quote   
  ----------------------------------------------------------------------*/
number  swap_fltois.quote(error_info option(nullable) error)
{	
	return swap_gen.quote(error);
}
/*-----------------------------------------------------------------------
  pay_fix  EXT SWAP: ok
  ----------------------------------------------------------------------*/
logical	 swap_fltois.pay_float(error_info option(nullable) error )
{
	SWAP_ENTRY_FUNC();
	return swap_gen.swap_payleg1(error);
}
/*-----------------------------------------------------------------------
  start_date	DLL_SWAP_HANDLER: ok
  ----------------------------------------------------------------------*/
date  swap_fltois.start_date(error_info option(nullable) error ) 
{
	return this.issue_date(error);
}
/*-----------------------------------------------------------------------
  notional	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.notional(	out number  notional_flt_leg, 
							out number  notional_ois_leg, 
							error_info option(nullable) error ) 
{	
	swap_gen.notional( notional_flt_leg,notional_ois_leg, error ) ;
	return;
}
/*-----------------------------------------------------------------------
  notional	EXT SWAP: ok
  ----------------------------------------------------------------------*/
number  swap_fltois.notional(error_info option(nullable) error ) 
{	
	return swap_gen.nominal(error);
}
/*-----------------------------------------------------------------------
  currency	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.currency(	out string  curr_flt_leg, 
							out string  curr_ois_leg, 
							error_info option(nullable) error) 
{	
	try{	
		instr_error_type t;
   		string 			s;	
		i().__swap_currency(curr_flt_leg,curr_ois_leg,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.currency");
		if(e)
			curr_flt_leg = curr_ois_leg = null<string>;
		return ;					
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.currency");		
		curr_flt_leg = curr_ois_leg = null<string>;
		return ;
	}
}
/*-----------------------------------------------------------------------
  currency(leg1- backward compat)	EXT SWAP: ok
  ----------------------------------------------------------------------*/
string  swap_fltois.currency(error_info option(nullable) error) option(hidden)
{	
	try{
		QL_FAIL_COND(ccy_swap_,"inapplicable function call for currency swaps (use 2-leg version)", this,true);
		
		instr_error_type t;
   		string 			s;	
		string c1,c2;
		i().__swap_currency(c1,c2,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.currency");	
		return e ? null<string>: c1;						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.currency");
		
		return null<string>;
	}
}
/*-----------------------------------------------------------------------
  reset_freq   	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.reset_freq(	out integer  reset_freq_flt_leg, 
								out integer  reset_freq_ois_leg, 
								error_info option(nullable) error) 
{	
	try{	
		error_info ee = error_info(true,true);
		ql_float_leg flt = this.float_leg(false,ee);
		QL_FAIL_COND(null(flt),"invalid swap_fltois", this, true);
			
		reset_freq_flt_leg = reset_freq_to_int(flt.float_def().reset_freq());
		reset_freq_ois_leg = 365;
		return ;						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.reset_freq");		
		reset_freq_flt_leg = reset_freq_ois_leg = err_freq();
		return ;
	}
}
/*-----------------------------------------------------------------------
  set_pay_leg   --> returns an external swap, a db swap cannot be changed in regards to pay vs receive 
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois.set_pay_leg(logical pay_float,							
									error_info option(nullable) error )
{
	try{
		SWAP_ENTRY_FUNC();
		__instrument  c = i().__swap_set_pay_leg1(pay_float );
		
		instr_error e = new instr_error();
		instrument cc 	= this.create( c, e, E_INIT);
		CORE_INT.instr_fail_check(null(cc), "invalid swap_fltois", this, e);
		
		if(!cc.is_valid(error)) 
			return null<swap_fltois>;	
				
		return dynamic_cast<swap_fltois>(cc);					
	}	
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.set_pay_leg");
		return null<swap_fltois>;
	}
}
/*-----------------------------------------------------------------------
  swap_leg
  ----------------------------------------------------------------------*/
vector(ql_swap_leg) swap_fltois.legs(logical clone,error_info option(nullable) error)
{	
	return swap_gen.swap_leg(clone,error);
}

/*-----------------------------------------------------------------------
  float_leg
  ----------------------------------------------------------------------*/
ql_float_leg swap_fltois.float_leg(logical clone,error_info option(nullable) error)
{	
	vector(ql_float_leg) l = swap_gen.swap_float_leg(clone,error);
	return !null(l) ? l[0] : null<ql_float_leg>;
}

/*-----------------------------------------------------------------------
  ois_leg
  ----------------------------------------------------------------------*/
ql_ois_leg swap_fltois.ois_leg(logical clone,error_info option(nullable) error)
{	
	vector(ql_ois_leg) l = swap_gen.swap_ois_leg(clone,error);
	return !null(l) ? l[0] : null<ql_ois_leg>;
}
/*-----------------------------------------------------------------------
  swap_fltois: ir_index
  ----------------------------------------------------------------------*/
ir_index swap_fltois.ir_index_flt_leg(error_info option(nullable) error)
{	
	return swap_gen.swap_ir_index_leg1(error);
}

/*-----------------------------------------------------------------------
  index_tenor_flat_leg
  ----------------------------------------------------------------------*/
string swap_fltois.index_tenor_flt_leg(error_info option(nullable) error) 
{	
	..ir_index ir = this.ir_index_flt_leg(error) ;
	if(error.is_error()) return null<string>;
	return ir.tenor();
}

/*-----------------------------------------------------------------------
  swap_fltois: ir_index
  ----------------------------------------------------------------------*/
ir_index swap_fltois.ir_index_ois_leg(error_info option(nullable) error)
{	
	return swap_gen.swap_ir_index_leg2(error);
}

/*-----------------------------------------------------------------------
  present_value
  ----------------------------------------------------------------------*/
number swap_fltois.present_value(	disc_func option(nullable)disc_func_flt_leg,
									disc_func option(nullable) 	disc_func_ois_leg,
									out number pv_flt_leg,
									out number pv_ois_leg,
									number option(nullable) notional,
									logical option(nullable) 	trade_date_pv,																								
									error_info option(nullable) error)
{	
	return swap_gen.present_value(disc_func_flt_leg,disc_func_ois_leg,pv_flt_leg,pv_ois_leg,notional,trade_date_pv,error);
}

number swap_fltois.present_value(	disc_func option(nullable)	disc_func_flt_leg,
									disc_func option(nullable) 	disc_func_ois_leg,
									out number pv_flt_leg,
									out number pv_ois_leg,
									date option(nullable) pv_date,
									date option(nullable) settle_date,
									logical  	incl_issue_cf,
									logical  	incl_mat_cf,	
									error_info option(nullable) error)
{	
	return swap_gen.present_value(disc_func_flt_leg,disc_func_ois_leg,pv_flt_leg,pv_ois_leg,pv_date, settle_date ,incl_issue_cf,incl_mat_cf,error );
}

number swap_fltois.present_value(	out number pv_flt_leg,
									out number pv_ois_leg,
									logical option(nullable) 	trade_date_pv,
									error_info option(nullable) error)
{	
	return swap_gen.present_value(pv_flt_leg,pv_ois_leg,trade_date_pv ,error );
}


/*-----------------------------------------------------------------------
  cash_flow_dates	EXT SWAP: ok   <PUBLIC>   interest payment dates, not principal pmt dates
  ----------------------------------------------------------------------*/
void  swap_fltois.cash_flow_dates(	logical 			post_settle, 
									out vector(date) 	pmt_dates_leg1,									
									out vector(date) 	pmt_dates_leg2,
									error_info option(nullable) error) 
{	
	try{	
		instr_error_type t;
   		string 			s;	
		i().__swap_ir_pmt_dates(post_settle,pmt_dates_leg1,pmt_dates_leg2, t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.cash_flow_dates");
		if(e) {
			pmt_dates_leg1 = pmt_dates_leg2 = null<vector(date)>;  
		}
		return;						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.cash_flow_dates");		
		pmt_dates_leg1 = pmt_dates_leg2 = null<vector(date)>; 
		return ;
	}
}
/*-----------------------------------------------------------------------
  prin_cash_flow_dates	EXT SWAP: ok   <PUBLIC>   
  ----------------------------------------------------------------------*/
void  swap_fltois.prin_cash_flow_dates(	logical 			post_settle, 
										out vector(date) 	pmt_dates_leg1,									
										out vector(date) 	pmt_dates_leg2,
										error_info option(nullable) error) 
{	
	try{	
		instr_error_type t;
   		string 			s;	
		i().__swap_prin_pmt_dates(post_settle,pmt_dates_leg1,pmt_dates_leg2, t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.prin_cash_flow_dates");
		if(e) {
			pmt_dates_leg1 = pmt_dates_leg2 = null<vector(date)>;  
		}
		return;						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.prin_cash_flow_dates");		
		pmt_dates_leg1 = pmt_dates_leg2 = null<vector(date)>; 
		return ;
	}
}

/*-----------------------------------------------------------------------
  accr_dates_flt_leg	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.accr_dates_flt_leg(logical 		post_settle, 
									out vector(date) 	accr_start_date,									
									out vector(date) 	accr_end_date,
									out vector(number)  accr_period,
									error_info option(nullable) 		error ) 
{	
	vector(date) accr_start_date_leg2;
	vector(date) accr_end_date_leg2;
	vector(number) accr_period_leg2;
	swap_gen.accr_dates(post_settle,
						accr_start_date,accr_end_date,accr_period,
						accr_start_date_leg2,accr_end_date_leg2,accr_period_leg2,
						error ) ;
	return;
}
/*-----------------------------------------------------------------------
  accr_dates_ois_leg	EXT SWAP: ok
  ----------------------------------------------------------------------*/
void  swap_fltois.accr_dates_ois_leg(	logical 			post_settle, 
										out vector(date) 	accr_start_date,									
										out vector(date) 	accr_end_date,
										out vector(number)  accr_period,
										error_info option(nullable) error ) 
{	
	vector(date) accr_start_date_leg1;
	vector(date) accr_end_date_leg1;
	vector(number) accr_period_leg1;
	swap_gen.accr_dates(post_settle,
						accr_start_date_leg1,accr_end_date_leg1,accr_period_leg1,
						accr_start_date,accr_end_date,accr_period,
						error ) ;
	return;
}
/*-----------------------------------------------------------------------
  is_endog_df	EXT SWAP: ok
  ----------------------------------------------------------------------*/
logical  swap_fltois.is_endog_df(error_info option(nullable) error ) 
{	
	try{
		SWAP_ENTRY_FUNC();
		error_info ee = error_info(true,true);		
		ql_ois_leg ois = this.ois_leg(false,ee);
		return ois.is_endog_df();						
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.is_endog_df");
		return null<logical >;
	}
}
/*-----------------------------------------------------------------------
  reset_freq_ois_leg	
  ----------------------------------------------------------------------*/
integer  swap_fltois.reset_freq_ois_leg(error_info option(nullable) error ) 
{
	return 365;
}
/*-----------------------------------------------------------------------
  currency_flt_leg	EXT SWAP: ok
  ----------------------------------------------------------------------*/
string swap_fltois.currency_flt_leg( error_info option(nullable) error ) 
{
	string c,  curr_flt_leg;
	swap_gen.currency(curr_flt_leg,  c,  error);
	return curr_flt_leg;

}
/*-----------------------------------------------------------------------
  currency_ois_leg	EXT SWAP: ok
  ----------------------------------------------------------------------*/
string swap_fltois.currency_ois_leg( error_info option(nullable) error ) 
{
	string c,  curr_ois_leg;
	swap_gen.currency(c,  curr_ois_leg,  error);
	return curr_ois_leg;

}
/*-----------------------------------------------------------------------
  dc_method   	EXT SWAP: ok
  ----------------------------------------------------------------------*/
day_count_method  swap_fltois.dc_method_flt_leg( error_info option(nullable) error ) 
{
	day_count_method d,dc_method_flt_leg;
	swap_gen.dc_method(dc_method_flt_leg,d,error);
	return dc_method_flt_leg;
}
/*-----------------------------------------------------------------------
  spread_ois_leg 	EXT SWAP: ok
  ----------------------------------------------------------------------*/
number  swap_fltois.spread_ois_leg(error_info option(nullable) error ) 
{	
	try{
		SWAP_ENTRY_FUNC();
		instr_error_type t;
   		string 			s;
		number q,quote_ois_leg;
		i().__swap_quote(q,quote_ois_leg,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.spread_ois_leg");
		if( e) {
			return null<number> ;
		}
		
		..quote_style qs = i().__swap_quote_style_e_leg2(t,s);
		e = CORE_INT.add_error_info(error,t,s, "swap_fltois.spread_ois_leg");
		if(e)
			return null<number>;
		
		switch(qs) {
			case ..quote_style.SPREAD:
				return quote_ois_leg;
			case ..quote_style.SPREAD_BP:					
			default:
				return quote_ois_leg/10000;
		}
							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.spread_ois_leg");
		return null<number> ;
	}
}
/*-----------------------------------------------------------------------
  spread_flt_leg 	EXT SWAP: ok
  ----------------------------------------------------------------------*/
number  swap_fltois.spread_flt_leg(error_info option(nullable) error ) 
{
	try{
		SWAP_ENTRY_FUNC();
		instr_error_type t;
   		string 			s;
		number q,quote_flt_leg;
		i().__swap_quote(quote_flt_leg,q,t, s);
		logical e = CORE_INT.add_error_info(error,t,s, "swap_fltois.spread_flt_leg");
		if( e) {
			return null<number> ;
		}
		
		..quote_style qs = i().__swap_quote_style_e_leg1(t,s);
		e = CORE_INT.add_error_info(error,t,s, "swap_fltois.spread_flt_leg");
		if(e)
			return null<number>;
		
		switch(qs) {
			case ..quote_style.SPREAD:
				return quote_flt_leg;
			case ..quote_style.SPREAD_BP:					
			default:
				return quote_flt_leg/10000;
		}
							
	}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.spread_flt_leg");
		return null<number> ;
	}
}
/*-----------------------------------------------------------------------
  swap_fltois: set_quote   EXT SWAP: ok
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois.set_quote(	number 	option(nullable) quote,									
									error_info option(nullable) error)
{
	SWAP_ENTRY_FUNC();
	instrument cc = instrument._set_quote(quote,null<date>, null<date>, false,error);
	return null(cc) ? null<swap_fltois>: dynamic_cast<swap_fltois>(cc);
}

/*-----------------------------------------------------------------------
  swap_fltois: set_quote   EXT SWAP: will throw because of quote_side
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois.set_quote(	string 	option(nullable) quote_side,
									error_info option(nullable) error)
{
	SWAP_ENTRY_FUNC();
	instrument cc = instrument._set_quote(quote_side,null<date>, null<date>, false,error);
	return null(cc) ? null<swap_fltois>: dynamic_cast<swap_fltois>(cc);
}

/*-----------------------------------------------------------------------
  fwd_func_ois_leg   EXT SWAP: ok
  ----------------------------------------------------------------------*/
fwd_func swap_fltois.fwd_func_ois_leg(error_info option(nullable) error )
{
	try{
		error_info  ee = new error_info(true,true);
		ql_ois_leg l = this.ois_leg(false,ee);
		return l.fwd_func();
		}
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.fwd_func_ois_leg");
		return null<fwd_func>;
	}
}

/*-----------------------------------------------------------------------
  set_spread_ois_leg
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois.set_spread_ois_leg(number option(nullable) spread ,
										   error_info option(nullable) error)
{
	try{
		
		QL_REQUIRE(ext_swap_ ,"function only available for non-db swaps" ,this,true );
		SWAP_ENTRY_FUNC();
		
		__instrument  c = i().__swap_ext_set_spread_leg2(spread);

		instr_error e = new instr_error();
		instrument cc 	= this.create( c, e, E_INIT);
		CORE_INT.instr_fail_check(null(cc), "invalid swap_fltois", this, e);
		
		if(!cc.is_valid(error)) 
			return null<swap_fltois>;	
				
		return dynamic_cast<swap_fltois>(cc);					
	}	
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.set_spread_ois_leg");
		return null<swap_fltois>;

	}
}
/*-----------------------------------------------------------------------
  set_spread_ois_leg
  ----------------------------------------------------------------------*/
swap_fltois swap_fltois.set_spread_flt_leg(number option(nullable) spread ,
										   error_info option(nullable) error)
{
	try{
		
		QL_REQUIRE(ext_swap_ ,"function only available for non-db swaps" ,this,true );
		SWAP_ENTRY_FUNC();
		
		__instrument  c = i().__swap_ext_set_spread_leg1(spread);

		instr_error e = new instr_error();
		instrument cc 	= this.create( c, e, E_INIT);
		CORE_INT.instr_fail_check(null(cc), "invalid swap_fltois", this, e);
		
		if(!cc.is_valid(error)) 
			return null<swap_fltois>;	
				
		return dynamic_cast<swap_fltois>(cc);					
	}	
	catch {
		CORE_INT.catch_error_info(error,err.type(),err.message(), "swap_fltois.set_spread_flt_leg");
		return null<swap_fltois>;

	}
}